#include "stdutils.h"
#include "complex.h"
#include "matrix.h"
#include "inout.h"
#include "func.h"

void
main(argc, argv)
int argc;
char **argv;

{
    char *input1_file_name = (char *) AllocateString(50);
    char *input2_file_name = (char *) AllocateString(50);
    char *output_file_name = (char *) AllocateString(50);
    int num_points;
    int height;
    int width;
    int zoom;
    int subtract;
    int input1_file_descriptor;
    int input2_file_descriptor;
    int output_file_descriptor;
    double *input1_data;
    double **input1_data_matrix;
    double *input2_data;
    double **input2_data_matrix;
    double *output_data;

    if(argc < 7) {
	(void) fprintf(stdout,
	"Syntax: addimage input1 input2 output height width zoom\n");
	(void) fflush(stdout);
	(void) exit(0);
    }
    (void) sscanf(argv[1], "%s", input1_file_name);
    (void) sscanf(argv[2], "%s", input2_file_name);
    (void) sscanf(argv[3], "%s", output_file_name);
    (void) sscanf(argv[4], "%d", &height);
    (void) sscanf(argv[5], "%d", &width);
    (void) sscanf(argv[6], "%d", &zoom);
    if(argc > 7) {
	subtract = 1;
    } else {
	subtract = 0;
    }
    input1_file_descriptor = (int) open(input1_file_name, O_RDONLY, 0);
    if((num_points = (int) lseek(input1_file_descriptor, 0, SEEK_END)) ==
    (int) NIL) {
	(void) fprintf(stdout, "Error: File %s nonexistent.  Exit 1.\n",
	input1_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) lseek(input1_file_descriptor, -(num_points), SEEK_END);
    input2_file_descriptor = (int) open(input2_file_name, O_RDONLY, 0);
    if((num_points = (int) lseek(input2_file_descriptor, 0, SEEK_END)) ==
    (int) NIL) {
	(void) fprintf(stdout, "Error: File %s nonexistent.  Exit 1.\n",
	input2_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) lseek(input2_file_descriptor, -(num_points), SEEK_END);
    output_file_descriptor = (int) creat(output_file_name, 0644);
    input1_data = (double *) RealCreateVector(num_points);
    input1_data_matrix = (double **) RealCreateMatrix(height, width);
    input2_data = (double *) RealCreateVector(num_points);
    input2_data_matrix = (double **) RealCreateMatrix(height, width);
    output_data = (double *) RealCreateVector(num_points);
    (void) ReadIn(input1_file_descriptor, input1_data, input1_data_matrix,
    NOVERTICALNOHORIZONTALFLIP, height, width, num_points);
    (void) ReadIn(input2_file_descriptor, input2_data, input2_data_matrix,
    NOVERTICALNOHORIZONTALFLIP, height, width, num_points);
    if(subtract) {
	(void) RealSubtractVectors(output_data,
	input1_data, input2_data, num_points);
    } else {
	(void) RealAddVectors(output_data,
	input1_data, input2_data, num_points);
    }
    (void) WriteOut(output_file_descriptor, NOVERTICALNOHORIZONTALFLIP, zoom,
    output_data, height, width, num_points, COEFF);
    if((int) close(input1_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, "Error: Cannot close input1 file.  Exit 1.\n");
	(void) fflush(stdout);
	(void) exit(1);
    }
    if((int) close(input2_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, "Error: Cannot close input2 file.  Exit 1.\n");
	(void) fflush(stdout);
	(void) exit(1);
    }
    if((int) close(output_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, "Error: Cannot close output file.  Exit 1.\n");
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) RealFreeVector(input1_data);
    (void) RealFreeMatrix(input1_data_matrix, height);
    (void) RealFreeVector(input2_data);
    (void) RealFreeMatrix(input2_data_matrix, height);
    (void) RealFreeVector(output_data);
    (void) free(input1_file_name);
    (void) free(input2_file_name);
    (void) free(output_file_name);
    (void) exit(0);
}
