#include "stdutils.h"
#include "complex.h"
#include "matrix.h"
#include "fft.h"
#include "inout.h"
#include "func.h"

void
main(argc, argv)
int argc;
char **argv;

{
    char *input_file_name = (char *) AllocateString(256);
    char *output_file_name = (char *) AllocateString(256);
    int input_file_descriptor;
    int output_file_descriptor;
    int num_points;
    int power2_points;
    double *input_data;
    double **input_data_matrix;
    double *output_data;
    COMPLEX **complex_input_data_vector;
    COMPLEX **complex_output_data_vector;
    int row;
    int col;

    if(argc < 4) {
	(void) fprintf(stdout, 
	"Syntax: dft1d infile outfile numpoints\n");
	(void) fflush(stdout);
	(void) exit(0);
    }
    (void) sscanf(argv[1], "%s", input_file_name);
    (void) sscanf(argv[2], "%s", output_file_name);
    (void) sscanf(argv[3], "%d", &num_points);
    input_file_descriptor = (int) open(input_file_name, O_RDONLY, 0);
    output_file_descriptor = (int) creat(output_file_name, 0644);
    if((num_points = (int) lseek(input_file_descriptor, 0, SEEK_END)) ==
    (int) NIL) {
	(void) fprintf(stdout, "Error: File %s nonexistent.  Exit 1.\n",
	input_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) lseek(input_file_descriptor, -(num_points), SEEK_END);
    for(power2_points = 1; power2_points < num_points; power2_points *= 2) {
	;
    }
    input_data = (double *) RealCreateVector(power2_points);
    input_data_matrix = (double **) RealCreateMatrix(1, power2_points);
    complex_input_data_vector =
    (COMPLEX **) ComplexCreateVector(power2_points);
    output_data = (double *) RealCreateVector(power2_points);
    complex_output_data_vector =
    (COMPLEX **) ComplexCreateVector(power2_points);
    (void) ReadIn(input_file_descriptor, input_data, input_data_matrix,
    NOVERTICALNOHORIZONTALFLIP, 1, num_points, num_points);
    (void) RealVectorToComplexVector(complex_input_data_vector, 
    input_data, power2_points);
    (void) DFT1D(complex_output_data_vector, 
    complex_input_data_vector, power2_points);
    (void) ComplexVectorToRealVector(output_data, 
    complex_output_data_vector, MAGNITUDE, power2_points);
    (void) WriteOut(output_file_descriptor, NOVERTICALNOHORIZONTALFLIP, 1, 
    output_data, 1, num_points, num_points, MAGCOEFF);
    if((int) close(input_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, 
	"Error: Cannot close input file '%s'.  Exit 1.\n", input_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    if((int) close(output_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, 
	"Error: Cannot close output file '%s'.  Exit 1.\n", output_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) RealFreeVector(input_data);
    (void) RealFreeMatrix(input_data_matrix, 1);
    (void) ComplexFreeVector(complex_input_data_vector, power2_points);
    (void) RealFreeVector(output_data);
    (void) ComplexFreeVector(complex_output_data_vector, power2_points);
    (void) free(input_file_name);
    (void) free(output_file_name);
    (void) exit(0);
}
