#include "stdutils.h"
#include "complex.h"
#include "matrix.h"
#include "inout.h"
#include "func.h"

#define ZOOM 2

#define NOOP 0
#define DOWN 1
#define UP 2
#define DOWNUP 3

void main();

void
main(argc, argv)
int argc;
char **argv;

{
    char *input_file_name = (char *) AllocateString(256);
    char *output_file_name = (char *) AllocateString(256);
    int input_file_descriptor;
    int output_file_descriptor;
    int height;
    int width;
    int command;
    int num_points;
    double *input_data;
    double **input_data_matrix;
    double *output_data;
    double **output_data_matrix;
    int down_height;
    int down_width;
    int down_points;
    int row;
    int col;
    int down_row;
    int down_col;
    double mean;

    if(argc < 5) {
	(void) fprintf(stdout, 
	"Syntax: downup infile outfile height width command\n");
	(void) fprintf(stdout,
	"Choices of command:\n");
	(void) fprintf(stdout,
	"   %d -> NO-OP;\n",
	NOOP);
	(void) fprintf(stdout,
	"   %d -> down by 2 in each direction;\n",
	DOWN);
	(void) fprintf(stdout,
	"   %d -> up by 2 in each direction;\n",
	UP);
	(void) fprintf(stdout,
	"   %d -> down by 2 followed by up by 2 in each direction;\n",
	DOWNUP);
	(void) fprintf(stdout,
	"Default is %d.\n",
	NOOP);
	(void) fflush(stdout);
	(void) exit(0);
    }
    (void) sscanf(argv[1], "%s", input_file_name);
    (void) sscanf(argv[2], "%s", output_file_name);
    (void) sscanf(argv[3], "%d", &height);
    (void) sscanf(argv[4], "%d", &width);
    if(argc > 5) {
	(void) sscanf(argv[5], "%d", &command);
    } else {
	command = NOOP;
    }
    if(!((command == NOOP) ||
    (command == DOWN) || (command == UP) || (command == DOWNUP))) {
	(void) fprintf(stdout,
	"Error: Illegal command %d.  Exit 1.\n",
	command);
	(void) fflush(stdout);
	(void) exit(1);
    }
    input_file_descriptor = (int) open(input_file_name, O_RDONLY, 0);
    output_file_descriptor = (int) creat(output_file_name, 0644);
    if((num_points = (int) lseek(input_file_descriptor, 0, SEEK_END)) ==
    (int) NIL) {
	(void) fprintf(stdout, "Error: File %s nonexistent.  Exit 1.\n",
	input_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) lseek(input_file_descriptor, -(num_points), SEEK_END);
    input_data = (double *) RealCreateVector(num_points);
    input_data_matrix = (double **) RealCreateMatrix(height, width);
    output_data = (double *) RealCreateVector(num_points);
    output_data_matrix = (double **) RealCreateMatrix(height, width);
    (void) ReadIn(input_file_descriptor, input_data, input_data_matrix,
    NOVERTICALNOHORIZONTALFLIP, height, width, num_points);
    (void) RealScaleMatrix(input_data_matrix, 
    input_data_matrix, 2.0 / (MAXGRAYVALUE - MINGRAYVALUE), height, width);
    for(row = 0; row < height; row++) {
	for(col = 0; col < width; col++) {
	    input_data_matrix[row][col] -= 1.0;
	}
    }
    (void) RealAccumulateMatrix(&mean, input_data_matrix, height, width);
    mean /= num_points;
printf("mean=%g\n",mean);fflush(stdout);
    down_height = height / ZOOM;
    down_width = width / ZOOM;
    down_points = down_height * down_width;
    for(down_row = row = 0; row < height; row++) {
	if(row % ZOOM) {
	    for(down_col = col = 0; col < width; col++) {
		if(col % ZOOM) {
		    output_data_matrix[down_row][down_col] =
		    input_data_matrix[row][col];
		    down_col++;
		}
	    }
	    down_row++;
	}
    }
    switch(command) {
	case NOOP:
	    (void) RealMatrixToRealVector(output_data, 
	    input_data_matrix, height, width);
	    (void) WriteOut(output_file_descriptor, NOVERTICALNOHORIZONTALFLIP,
	    1, output_data, height, width, num_points, COEFF);
	break;
	case DOWN:
	    (void) RealMatrixToRealVector(output_data, 
	    output_data_matrix, down_height, down_width);
	    (void) WriteOut(output_file_descriptor, NOVERTICALNOHORIZONTALFLIP,
	    1, output_data, down_height, down_width, down_points, COEFF);
	break;
	case UP:
	    (void) RealMatrixToRealVector(output_data, 
	    input_data_matrix, height, width);
	    (void) WriteOut(output_file_descriptor, NOVERTICALNOHORIZONTALFLIP,
	    ZOOM, output_data, height, width, num_points, COEFF);
	break;
	case DOWNUP:
	    (void) RealMatrixToRealVector(output_data, 
	    output_data_matrix, down_height, down_width);
	    (void) WriteOut(output_file_descriptor, NOVERTICALNOHORIZONTALFLIP,
	    ZOOM, output_data, down_height, down_width, down_points, COEFF);
	break;
	default:
	break;
    }
    if((int) close(input_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, 
	"Error: Cannot close input file '%s'.  Exit 1.\n", input_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    if((int) close(output_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, 
	"Error: Cannot close output file '%s'.  Exit 1.\n",
	output_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) RealFreeVector(input_data);
    (void) RealFreeMatrix(input_data_matrix, height);
    (void) RealFreeVector(output_data);
    (void) RealFreeMatrix(output_data_matrix, height);
    (void) free(input_file_name);
    (void) free(output_file_name);
    (void) exit(0);
}
