#include "stdutils.h"
#include "complex.h"
#include "matrix.h"
#include "inout.h"
#include "func.h"

#define EXTENT 5
#define HMASK (2 * EXTENT + 1)
#define WMASK HMASK
#define MASKSIZE (HMASK * WMASK)

#define DVAR (2.0 * (1.0) * (1.0))

void
main(argc, argv)
int argc;
char **argv;

{
    char *input_file_name = (char *) AllocateString(256);
    char *output_file_name = (char *) AllocateString(256);
    int input_file_descriptor;
    int output_file_descriptor;
    int height;
    int width;
    int num_points;
    double *input_data;
    double **input_data_matrix;
    double *output_data;
    double **output_data_matrix;
    double **img1;
    double **img2;
    int row;
    int col;
    int zoom;
    double **lpf;
    double **bpf;
    double exponent;
    double sum;
    double tmp;
    double negative;
    double positive;
    int n2;
    int n1;

    if(argc < 6) {
	(void) fprintf(stdout, 
	"Syntax: edges infile outfile height width zoom\n");
	(void) fflush(stdout);
	(void) exit(0);
    }
    (void) sscanf(argv[1], "%s", input_file_name);
    (void) sscanf(argv[2], "%s", output_file_name);
    (void) sscanf(argv[3], "%d", &height);
    (void) sscanf(argv[4], "%d", &width);
    (void) sscanf(argv[5], "%d", &zoom);
    input_file_descriptor = (int) open(input_file_name, O_RDONLY, 0);
    output_file_descriptor = (int) creat(output_file_name, 0644);
    if((num_points = (int) lseek(input_file_descriptor, 0, SEEK_END)) ==
    (int) NIL) {
	(void) fprintf(stdout, "Error: File %s nonexistent.  Exit 1.\n",
	input_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) lseek(input_file_descriptor, -(num_points), SEEK_END);
    input_data = (double *) RealCreateVector(num_points);
    input_data_matrix = (double **) RealCreateMatrix(height, width);
    output_data = (double *) RealCreateVector(num_points);
    output_data_matrix = (double **) RealCreateMatrix(height, width);
    img1 = (double **) RealCreateMatrix(height, width);
    img2 = (double **) RealCreateMatrix(height, width);
    lpf = (double **) RealCreateMatrix(HMASK, WMASK);
    bpf = (double **) RealCreateMatrix(HMASK, WMASK);
    (void) ReadIn(input_file_descriptor, input_data, input_data_matrix,
    NOVERTICALNOHORIZONTALFLIP, height, width, num_points);
    for(n2 = -EXTENT; n2 <= EXTENT; n2++) {
	for(n1 = -EXTENT; n1 <= EXTENT; n1++) {
	    exponent = 1.0 * ((n2 * n2) + (n1 * n1)) / (DVAR + EPSILON);
	    lpf[(n2+HMASK)%HMASK][(n1+WMASK)%WMASK] = (double) exp(-exponent);
	}
    }
    (void) RealAccumulateMatrix(&sum, lpf, HMASK, WMASK);
    (void) RealScaleMatrix(lpf, lpf, (1.0 / (sum + EPSILON)), HMASK, WMASK);
    n2 = -1;
    n1 = -1;
    bpf[(n2+HMASK)%HMASK][(n1+WMASK)%WMASK] = 1.0;
    n1 = 0;
    bpf[(n2+HMASK)%HMASK][(n1+WMASK)%WMASK] = (-2.0);
    n1 = 1;
    bpf[(n2+HMASK)%HMASK][(n1+WMASK)%WMASK] = 1.0;
    n2 = 0;
    n1 = -1;
    bpf[(n2+HMASK)%HMASK][(n1+WMASK)%WMASK] = (-2.0);
    n1 = 0;
    bpf[(n2+HMASK)%HMASK][(n1+WMASK)%WMASK] = 4.0;
    n1 = 1;
    bpf[(n2+HMASK)%HMASK][(n1+WMASK)%WMASK] = (-2.0);
    n2 = 1;
    n1 = -1;
    bpf[(n2+HMASK)%HMASK][(n1+WMASK)%WMASK] = 1.0;
    n1 = 0;
    bpf[(n2+HMASK)%HMASK][(n1+WMASK)%WMASK] = (-2.0);
    n1 = 1;
    bpf[(n2+HMASK)%HMASK][(n1+WMASK)%WMASK] = 1.0;
    for(row = 0; row < height; row++) {
	for(col = 0; col < width; col++) {
	    tmp = 0.0;
	    for(n2 = -EXTENT; n2 <= EXTENT; n2++) {
		for(n1 = -EXTENT; n1 <= EXTENT; n1++) {
		    tmp +=
		    lpf[(n2+HMASK)%HMASK][(n1+WMASK)%WMASK] *
		    input_data_matrix
		    [(row-n2+height)%height][(col-n1+width)%width];
		}
	    }
	    img1[row][col] = tmp;
	}
    }
    for(row = 0; row < height; row++) {
	for(col = 0; col < width; col++) {
	    tmp = 0.0;
	    for(n2 = -EXTENT; n2 <= EXTENT; n2++) {
		for(n1 = -EXTENT; n1 <= EXTENT; n1++) {
		    tmp +=
		    bpf[(n2+HMASK)%HMASK][(n1+WMASK)%WMASK] *
		    img1[(row-n2+height)%height][(col-n1+width)%width];
		}
	    }
	    img2[row][col] = tmp;
	}
    }
    for(row = 0; row < height; row++) {
	for(col = 0; col < width; col++) {
	    if(img2[row][col] == 0.0) {
		output_data_matrix[row][col] = 1.0;
	    } else {
		negative = positive = 0.0;
		for(n2 = -1; n2 <= 1; n2++) {
		    for(n1 = -1; n1 <= 1; n1++) {
			if((tmp =
			img2[(row-n2+height)%height][(col-n1+width)%width]) <
			0.0) {
			    negative += tmp;
			} else {
			    positive += tmp;
			}
		    }
		}
		if((negative < 0) && (positive > 0) &&
		((double) AbsFloat(negative) > 0.0e1) && (positive > 0.0e1)) {
		    output_data_matrix[row][col] = 1.0;
		} else {
		    output_data_matrix[row][col] = (-1.0);
		}
	    }
	}
    }
    (void) RealMatrixToRealVector(output_data, 
    output_data_matrix, height, width);
/*
RealMatrixToRealVector(output_data, img1, height, width);
*/
    (void) WriteOut(output_file_descriptor, NOVERTICALNOHORIZONTALFLIP, zoom, 
    output_data, height, width, num_points, COEFF);
    if((int) close(input_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, 
	"Error: Cannot close input file '%s'.  Exit 1.\n", input_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    if((int) close(output_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, 
	"Error: Cannot close output file '%s'.  Exit 1.\n",
	output_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) RealFreeVector(input_data);
    (void) RealFreeMatrix(input_data_matrix, height);
    (void) RealFreeVector(output_data);
    (void) RealFreeMatrix(output_data_matrix, height);
    (void) RealFreeMatrix(img1, height);
    (void) RealFreeMatrix(img2, height);
    (void) RealFreeMatrix(lpf, HMASK);
    (void) RealFreeMatrix(bpf, HMASK);
    (void) free(input_file_name);
    (void) free(output_file_name);
    (void) exit(0);
}
