#include "stdutils.h"
#include "complex.h"
#include "matrix.h"
#include "inout.h"
#include "func.h"

#define EXTENT 15
#define HMASK (2 * EXTENT + 1)
#define WMASK HMASK
#define MASKSIZE (HMASK * WMASK)

#define DVAR ((1.0 * (2 * EXTENT + 1) * (2 * EXTENT + 1)) / 3.6e1)

void
main(argc, argv)
int argc;
char **argv;

{
    char *input_file_name = (char *) AllocateString(256);
    char *output_file_name = (char *) AllocateString(256);
    int input_file_descriptor;
    int output_file_descriptor;
    int height;
    int width;
    int num_points;
    double *input_data;
    double **input_data_matrix;
    double *output_data;
    double **output_data_matrix;
    double **img1;
    double **img2;
    int row;
    int col;
    int zoom;
    double mean;
    double **lpf;
    double **bpf;
    double exponent;
    double factor;
    double sum;
    double tmp;
    int n2;
    int n1;

    if(argc < 6) {
	(void) fprintf(stdout, 
	"Syntax: edges infile outfile height width zoom\n");
	(void) fflush(stdout);
	(void) exit(0);
    }
    (void) sscanf(argv[1], "%s", input_file_name);
    (void) sscanf(argv[2], "%s", output_file_name);
    (void) sscanf(argv[3], "%d", &height);
    (void) sscanf(argv[4], "%d", &width);
    (void) sscanf(argv[5], "%d", &zoom);
    input_file_descriptor = (int) open(input_file_name, O_RDONLY, 0);
    output_file_descriptor = (int) creat(output_file_name, 0644);
    if((num_points = (int) lseek(input_file_descriptor, 0, SEEK_END)) ==
    (int) NIL) {
	(void) fprintf(stdout, "Error: File %s nonexistent.  Exit 1.\n",
	input_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) lseek(input_file_descriptor, -(num_points), SEEK_END);
    input_data = (double *) RealCreateVector(num_points);
    input_data_matrix = (double **) RealCreateMatrix(height, width);
    output_data = (double *) RealCreateVector(num_points);
    output_data_matrix = (double **) RealCreateMatrix(height, width);
    img1 = (double **) RealCreateMatrix(height, width);
    img2 = (double **) RealCreateMatrix(height, width);
    lpf = (double **) RealCreateMatrix(HMASK, WMASK);
    bpf = (double **) RealCreateMatrix(HMASK, WMASK);
    (void) ReadIn(input_file_descriptor, input_data, input_data_matrix,
    NOVERTICALNOHORIZONTALFLIP, height, width, num_points);
    (void) RealScaleMatrix(input_data_matrix,
    input_data_matrix, 2.0 / (MAXGRAYVALUE - MINGRAYVALUE), height, width);
    for(row = 0; row < height; row++) {
	for(col = 0; col < width; col++) {
	    input_data_matrix[row][col] -= 1.0;
	}
    }
    (void) RealAccumulateMatrix(&mean, input_data_matrix, height, width);
    mean /= num_points;
printf("mean=%g\n",mean);fflush(stdout);
    (void) fprintf(stdout,
    "%d by %d window with 2*sigma^2=%g\n",
    HMASK, WMASK, DVAR);
    (void) fflush(stdout);
    /**/
    for(row = 0; row < height; row++) {
	for(col = 0; col < width; col++) {
	    mean = 0.0;
	    for(n2 = -2; n2 <= 2; n2++) {
		for(n1 = -2; n1 <= 2; n1++) {
		    mean +=
		    input_data_matrix
		    [(row-n2+height)%height][(col-n1+width)%width];
		}
	    }
	    mean /= 25;
	    sum = 0.0;
	    for(n2 = -2; n2 <= 2; n2++) {
		for(n1 = -2; n1 <= 2; n1++) {
		    sum +=
		    (input_data_matrix
		    [(row-n2+height)%height][(col-n1+width)%width] - mean) *
		    (input_data_matrix
		    [(row-n2+height)%height][(col-n1+width)%width] - mean);
		}
	    }
	    sum /= 24;
	    for(n2 = -EXTENT; n2 <= EXTENT; n2++) {
		for(n1 = -EXTENT; n1 <= EXTENT; n1++) {
		    /*
		    exponent =
		    1.0 * ((n2 * n2) + (n1 * n1)) / (DVAR + EPSILON);
		    */
		    /**/
		    exponent =
		    1.0 * ((n2 * n2) + (n1 * n1)) / (DVAR + sum + EPSILON);
		    /**/
		    factor = 1.0 - exponent;
		    bpf[(n2+HMASK)%HMASK][(n1+WMASK)%WMASK] =
		    factor * (double) exp(-exponent);
		}
	    }
	    tmp = 0.0;
	    for(n2 = -EXTENT; n2 <= EXTENT; n2++) {
		for(n1 = -EXTENT; n1 <= EXTENT; n1++) {
		    tmp +=
		    bpf[(n2+HMASK)%HMASK][(n1+WMASK)%WMASK] *
		    input_data_matrix
		    [(row-n2+height)%height][(col-n1+width)%width];
		}
	    }
	    img1[row][col] = tmp;
	}
    }
    /**/
    /*
    for(n1 = -EXTENT; n1 <= EXTENT; n1++) {
	exponent = 1.0 * (n1 * n1) / (DVAR + EPSILON);
	factor = 1.0 - 2.0 * exponent;
	lpf[0][(n1+WMASK)%WMASK] = (double) exp(-exponent);
	bpf[0][(n1+WMASK)%WMASK] = factor * (double) exp(-exponent);
    }
    */
    /*
    for(row = 0; row < height; row++) {
	for(col = 0; col < width; col++) {
	    tmp = 0.0;
	    for(n1 = -EXTENT; n1 <= EXTENT; n1++) {
		tmp +=
		bpf[0][(n1+WMASK)%WMASK] *
		input_data_matrix[row][(col-n1+width)%width];
	    }
	    img1[row][col] = tmp;
	}
    }
    for(col = 0; col < width; col++) {
	for(row = 0; row < height; row++) {
	    tmp = 0.0;
	    for(n2 = -EXTENT; n2 <= EXTENT; n2++) {
		tmp +=
		lpf[0][(n2+HMASK)%HMASK] * img1[(row-n2+height)%height][col];
	    }
	    img2[row][col] = tmp;
	}
    }
    for(col = 0; col < width; col++) {
	for(row = 0; row < height; row++) {
	    tmp = 0.0;
	    for(n2 = -EXTENT; n2 <= EXTENT; n2++) {
		tmp +=
		bpf[0][(n2+HMASK)%HMASK] *
		input_data_matrix[(row-n2+height)%height][col];
	    }
	    output_data_matrix[row][col] = tmp;
	}
    }
    for(row = 0; row < height; row++) {
	for(col = 0; col < width; col++) {
	    tmp = 0.0;
	    for(n1 = -EXTENT; n1 <= EXTENT; n1++) {
		tmp +=
		lpf[0][(n1+WMASK)%WMASK] *
		output_data_matrix[row][(col-n1+width)%width];
	    }
	    img1[row][col] = tmp + img2[row][col];
	}
    }
    */
    for(row = 0; row < height; row++) {
	for(col = 0; col < width; col++) {
	    output_data_matrix[row][col] = (-1.0);
	}
    }
    for(row = 0; row < height; row++) {
	for(col = 0; col < width; col++) {
	    if(!(int) FixFloat((double) AbsFloat(img1[row][col]))) {
		output_data_matrix[row][col] = 1.0;
	    } else {
		n2 = -1;
		n1 = -1;
		sum = img1[(row-n2+height)%height][(col-n1+width)%width];
		n2 = 1;
		n1 = 1;
		tmp = img1[(row-n2+height)%height][(col-n1+width)%width];
		if(!(int) FixFloat((double) SignFloat(sum) +
		(double) SignFloat(tmp))) {
		    output_data_matrix[row][col] = 1.0;
		}
		n2 = -1;
		n1 = 1;
		sum = img1[(row-n2+height)%height][(col-n1+width)%width];
		n2 = 1;
		n1 = -1;
		tmp = img1[(row-n2+height)%height][(col-n1+width)%width];
		if(!(int) FixFloat((double) SignFloat(sum) +
		(double) SignFloat(tmp))) {
		    output_data_matrix[row][col] = 1.0;
		}
		n2 = 0;
		n1 = -1;
		sum = img1[(row-n2+height)%height][(col-n1+width)%width];
		n2 = 0;
		n1 = 1;
		tmp = img1[(row-n2+height)%height][(col-n1+width)%width];
		if(!(int) FixFloat((double) SignFloat(sum) +
		(double) SignFloat(tmp))) {
		    output_data_matrix[row][col] = 1.0;
		}
		n2 = -1;
		n1 = 0;
		sum = img1[(row-n2+height)%height][(col-n1+width)%width];
		n2 = 1;
		n1 = 0;
		tmp = img1[(row-n2+height)%height][(col-n1+width)%width];
		if(!(int) FixFloat((double) SignFloat(sum) +
		(double) SignFloat(tmp))) {
		    output_data_matrix[row][col] = 1.0;
		}
	    }
	}
    }
    (void) RealMatrixToRealVector(output_data, 
    output_data_matrix, height, width);
/*
RealMatrixToRealVector(output_data, img1, height, width);
*/
    (void) WriteOut(output_file_descriptor, NOVERTICALNOHORIZONTALFLIP, zoom, 
    output_data, height, width, num_points, COEFF);
    if((int) close(input_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, 
	"Error: Cannot close input file '%s'.  Exit 1.\n", input_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    if((int) close(output_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, 
	"Error: Cannot close output file '%s'.  Exit 1.\n",
	output_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) RealFreeVector(input_data);
    (void) RealFreeMatrix(input_data_matrix, height);
    (void) RealFreeVector(output_data);
    (void) RealFreeMatrix(output_data_matrix, height);
    (void) RealFreeMatrix(img1, height);
    (void) RealFreeMatrix(img2, height);
    (void) RealFreeMatrix(lpf, HMASK);
    (void) RealFreeMatrix(bpf, HMASK);
    (void) free(input_file_name);
    (void) free(output_file_name);
    (void) exit(0);
}
