#include "stdutils.h"
#include "errorimage.h"
#include "complex.h"
#include "matrix.h"
#include "inout.h"
#include "func.h"

/*
The main function for finding the error between two images.
*/

void
main(argc, argv)
int argc;
char **argv;

{
    char *orig_file_name = (char *) AllocateString(50);
    char *approx_file_name = (char *) AllocateString(50);
    char *error_file_name = (char *) AllocateString(50);
    int num_points;
    int height;
    int width;
    int zoom;
    int orig_file_descriptor;
    int approx_file_descriptor;
    int error_file_descriptor;
    double *orig_data;
    double **orig_data_matrix;
    double *approx_data;
    double **approx_data_matrix;
    double *error_data;

    if(argc < 7) {
	(void) fprintf(stdout,
	"Syntax: errorimage orig approx error height width zoom\n");
	(void) fflush(stdout);
	(void) exit(0);
    }
    (void) sscanf(argv[1], "%s", orig_file_name);
    (void) sscanf(argv[2], "%s", approx_file_name);
    (void) sscanf(argv[3], "%s", error_file_name);
    (void) sscanf(argv[4], "%d", &height);
    (void) sscanf(argv[5], "%d", &width);
    (void) sscanf(argv[6], "%d", &zoom);
    orig_file_descriptor = (int) open(orig_file_name, O_RDONLY, 0);
    if((num_points = (int) lseek(orig_file_descriptor, 0, SEEK_END)) ==
    (int) NIL) {
	(void) fprintf(stdout, "Error: File %s nonexistent.  Exit 1.\n",
	orig_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) lseek(orig_file_descriptor, -(num_points), SEEK_END);
    approx_file_descriptor = (int) open(approx_file_name, O_RDONLY, 0);
    if((num_points = (int) lseek(approx_file_descriptor, 0, SEEK_END)) ==
    (int) NIL) {
	(void) fprintf(stdout, "Error: File %s nonexistent.  Exit 1.\n",
	approx_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) lseek(approx_file_descriptor, -(num_points), SEEK_END);
    error_file_descriptor = (int) creat(error_file_name, 0644);
    orig_data = (double *) RealCreateVector(num_points);
    orig_data_matrix = (double **) RealCreateMatrix(height, width);
    approx_data = (double *) RealCreateVector(num_points);
    approx_data_matrix = (double **) RealCreateMatrix(height, width);
    error_data = (double *) RealCreateVector(num_points);
    (void) ReadIn(orig_file_descriptor, orig_data, orig_data_matrix,
    NOVERTICALNOHORIZONTALFLIP, height, width, num_points);
    (void) ReadIn(approx_file_descriptor, approx_data, approx_data_matrix,
    NOVERTICALNOHORIZONTALFLIP, height, width, num_points);
    (void) RealSubtractVectors(error_data, orig_data, approx_data, num_points);
    (void) WriteOut(error_file_descriptor, NOVERTICALNOHORIZONTALFLIP, zoom,
    error_data, height, width, num_points, COEFF);
    if((int) close(orig_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, "Error: Cannot close orig file.  Exit 1.\n");
	(void) fflush(stdout);
	(void) exit(1);
    }
    if((int) close(approx_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, "Error: Cannot close approx file.  Exit 1.\n");
	(void) fflush(stdout);
	(void) exit(1);
    }
    if((int) close(error_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, "Error: Cannot close error file.  Exit 1.\n");
	(void) fflush(stdout);
	(void) exit(1);
    }
    orig_file_descriptor = (int) open(orig_file_name, O_RDONLY, 0);
    if((num_points = (int) lseek(orig_file_descriptor, 0, SEEK_END)) ==
    (int) NIL) {
	(void) fprintf(stdout, "Error: File %s nonexistent.  Exit 1.\n",
	orig_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) lseek(orig_file_descriptor, -(num_points), SEEK_END);
    approx_file_descriptor = (int) open(approx_file_name, O_RDONLY, 0);
    if((num_points = (int) lseek(approx_file_descriptor, 0, SEEK_END)) ==
    (int) NIL) {
	(void) fprintf(stdout, "Error: File %s nonexistent.  Exit 1.\n",
	approx_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) lseek(approx_file_descriptor, -(num_points), SEEK_END);
    error_file_descriptor = (int) creat(error_file_name, 0644);
    (void) WriteOut(error_file_descriptor, NOVERTICALNOHORIZONTALFLIP, zoom,
    error_data, height, width, num_points, MAGCOEFF);
    if((int) close(orig_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, "Error: Cannot close orig file.  Exit 1.\n");
	(void) fflush(stdout);
	(void) exit(1);
    }
    if((int) close(approx_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, "Error: Cannot close approx file.  Exit 1.\n");
	(void) fflush(stdout);
	(void) exit(1);
    }
    if((int) close(error_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, "Error: Cannot close error file.  Exit 1.\n");
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) RealFreeVector(orig_data);
    (void) RealFreeMatrix(orig_data_matrix, height);
    (void) RealFreeVector(approx_data);
    (void) RealFreeMatrix(approx_data_matrix, height);
    (void) free(orig_file_name);
    (void) free(approx_file_name);
    (void) free(error_file_name);
    (void) exit(0);
}
