#include "stdutils.h"
#include "complex.h"
#include "matrix.h"
#include "inout.h"
#include "func.h"

#define EXTENT 2
#define HMASK (2 * EXTENT + 1)
#define WMASK HMASK
#define MASKSIZE (HMASK * WMASK)

#define DVARMAX (5.0e-1 * EXTENT * EXTENT)/*double maximum variance*/
#define HALFDVARMAX (5.0e-1 * DVARMAX)/*half double maximum variance*/

void main();

void
main(argc, argv)
int argc;
char **argv;

{
    char *input_file_name = (char *) AllocateString(256);
    char *output_file_name = (char *) AllocateString(256);
    int input_file_descriptor;
    int output_file_descriptor;
    int height;
    int width;
    int zoom;
    int num_points;
    double *input_data;
    double **input_data_matrix;
    double *output_data;
    double **output_data_matrix;
    double **lpf;
    int extent;
    int hmask;
    int wmask;
    int masksize;
    double dvarmax;
    double halfdvarmax;
    int row;
    int col;
    double theta;
    double c;
    double s;
    double mag;
    double vary;
    double varx;
    double exponent;
    int n2;
    int n1;
    double sum;
    double tmp;
    double mean;

    if(argc < 6) {
	(void) fprintf(stdout, 
	"Syntax: gauss infile outfile height width zoom [extent] [sigma]\n");
	(void) fprintf(stdout, 
	"Default extent is %d.\n",
	EXTENT);
	(void) fprintf(stdout, 
	"Default sigma is %g.\n",
	(double) sqrt(HALFDVARMAX));
	(void) fflush(stdout);
	(void) exit(0);
    }
    (void) sscanf(argv[1], "%s", input_file_name);
    (void) sscanf(argv[2], "%s", output_file_name);
    (void) sscanf(argv[3], "%d", &height);
    (void) sscanf(argv[4], "%d", &width);
    (void) sscanf(argv[5], "%d", &zoom);
    extent = EXTENT;
    dvarmax = DVARMAX;
    if(argc > 6) {
	(void) sscanf(argv[6], "%d", &extent);
    }
    if(argc > 7) {
	(void) sscanf(argv[7], "%F", &dvarmax);
	dvarmax = 2.0 * dvarmax * dvarmax;
    }
    halfdvarmax = 5.0e-1 * dvarmax;
    hmask = (2 * extent + 1);
    wmask = hmask;
    masksize = (hmask * wmask);
    (void) fprintf(stdout, 
    "Convolving with an isotropic %d by %d Gaussian low-pass filter with the ",
    hmask, wmask);
    (void) fprintf(stdout, 
    "standard\n");
    (void) fprintf(stdout,
    "deviation of %g.\n",
    (double) sqrt(halfdvarmax));
    input_file_descriptor = (int) open(input_file_name, O_RDONLY, 0);
    output_file_descriptor = (int) creat(output_file_name, 0644);
    if((num_points = (int) lseek(input_file_descriptor, 0, SEEK_END)) ==
    (int) NIL) {
	(void) fprintf(stdout, "Error: File %s nonexistent.  Exit 1.\n",
	input_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) lseek(input_file_descriptor, -(num_points), SEEK_END);
    input_data = (double *) RealCreateVector(num_points);
    input_data_matrix = (double **) RealCreateMatrix(height, width);
    output_data = (double *) RealCreateVector(num_points);
    output_data_matrix = (double **) RealCreateMatrix(height, width);
    lpf = (double **) RealCreateMatrix(hmask, wmask);
    (void) ReadIn(input_file_descriptor, input_data, input_data_matrix,
    NOVERTICALNOHORIZONTALFLIP, height, width, num_points);
    /*
    (void) RealScaleMatrix(input_data_matrix, 
    input_data_matrix, 2.0 / (MAXGRAYVALUE - MINGRAYVALUE), height, width);
    for(row = 0; row < height; row++) {
	for(col = 0; col < width; col++) {
	    input_data_matrix[row][col] -= 1.0;
	}
    }
    */
    (void) RealAccumulateMatrix(&mean, input_data_matrix, height, width);
    mean /= num_points;
printf("mean=%g\n",mean);fflush(stdout);
    theta = 0.0;
    c = (double) cos(theta);
    s = (double) sin(theta);
    mag = 0.0;
    vary = halfdvarmax * (1.0 - mag);
    varx = dvarmax - vary;
    for(n2 = -extent; n2 <= extent; n2++) {
	for(n1 = -extent; n1 <= extent; n1++) {
	    exponent = 
	    ((s * n1 + c * n2) * (s * n1 + c * n2) /
	    (vary + EPSILON)) +
	    ((c * n1 - s * n2) * (c * n1 - s * n2) /
	    (varx + EPSILON));
	    lpf[(n2+hmask)%hmask][(n1+wmask)%wmask] =
	    (double) exp(-exponent);
	}
    }
    (void) RealAccumulateMatrix(&sum, lpf, hmask, wmask);
    (void) RealScaleMatrix(lpf, lpf, (1.0 / (sum + EPSILON)), hmask, wmask);
    for(row = 0; row < height; row++) {
	for(col = 0; col < width; col++) {
	    tmp = 0.0;
	    for(n2 = -extent; n2 <= extent; n2++) {
		for(n1 = -extent; n1 <= extent; n1++) {
		    tmp +=
		    lpf[(n2+hmask)%hmask][(n1+wmask)%wmask] * input_data_matrix
		    [(row-n2+height)%height][(col-n1+width)%width];
		}
	    }
	    output_data_matrix[row][col] = tmp;
	}
    }
    (void) RealMatrixToRealVector(output_data, 
    output_data_matrix, height, width);
    (void) WriteOut(output_file_descriptor, NOVERTICALNOHORIZONTALFLIP, zoom, 
    output_data, height, width, num_points, SIGNAL);
    if((int) close(input_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, 
	"Error: Cannot close input file '%s'.  Exit 1.\n", input_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    if((int) close(output_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, 
	"Error: Cannot close output file '%s'.  Exit 1.\n",
	output_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) RealFreeVector(input_data);
    (void) RealFreeMatrix(input_data_matrix, height);
    (void) RealFreeVector(output_data);
    (void) RealFreeMatrix(output_data_matrix, height);
    (void) RealFreeMatrix(lpf, hmask);
    (void) free(input_file_name);
    (void) free(output_file_name);
    (void) exit(0);
}
