#include "stdutils.h"
#include "complex.h"
#include "matrix.h"
#include "inout.h"
#include "func.h"

#define NUMGRAYS (MAXGRAYVALUE - MINGRAYVALUE + 1)
#define EQUALIZATION 0
#define EXTREMES 1
#define MIDDLE 2

void
main(argc, argv)
int argc;
char **argv;

{
    char *input_file_name = (char *) AllocateString(256);
    char *output_file_name = (char *) AllocateString(256);
    int input_file_descriptor;
    int output_file_descriptor;
    int height;
    int width;
    int num_points;
    double *input_data;
    double **input_data_matrix;
    double *output_data;
    double **output_data_matrix;
    int row;
    int col;
    int zoom;
    int modification;
    int *histogram;
    int *cumulative;
    int *desired;
    int *distribution;
    unsigned char *clut;
    int midgray;
    int level;
    int fill;
    int found;
    int sum;

    if(argc < 6) {
	(void) fprintf(stdout, 
	"Syntax: histeq infile outfile height width zoom modification\n");
	(void) fprintf(stdout, 
	"Choices of modification:\n");
	(void) fprintf(stdout,
	"   %d -> equalization;\n",
	EQUALIZATION);
	(void) fprintf(stdout,
	"   %d -> extremes;\n",
	EXTREMES);
	(void) fprintf(stdout,
	"   %d -> middle;\n",
	MIDDLE);
	(void) fprintf(stdout,
	"Default is %d.\n",
	EQUALIZATION);
	(void) fflush(stdout);
	(void) exit(0);
    }
    (void) sscanf(argv[1], "%s", input_file_name);
    (void) sscanf(argv[2], "%s", output_file_name);
    (void) sscanf(argv[3], "%d", &height);
    (void) sscanf(argv[4], "%d", &width);
    (void) sscanf(argv[5], "%d", &zoom);
    if(argc > 6) {
	(void) sscanf(argv[6], "%d", &modification);
    } else {
	modification = EQUALIZATION;
    }
    if(!((modification == EQUALIZATION) ||
    (modification == EXTREMES) || (modification == MIDDLE))) {
	(void) fprintf(stdout,
	"Error: Illegal histogram modification %d.  Exit 1.\n",
	modification);
	(void) fflush(stdout);
	(void) exit(1);
    }
    input_file_descriptor = (int) open(input_file_name, O_RDONLY, 0);
    output_file_descriptor = (int) creat(output_file_name, 0644);
    if((num_points = (int) lseek(input_file_descriptor, 0, SEEK_END)) ==
    (int) NIL) {
	(void) fprintf(stdout, "Error: File %s nonexistent.  Exit 1.\n",
	input_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) lseek(input_file_descriptor, -(num_points), SEEK_END);
    input_data = (double *) RealCreateVector(num_points);
    input_data_matrix = (double **) RealCreateMatrix(height, width);
    output_data = (double *) RealCreateVector(num_points);
    output_data_matrix = (double **) RealCreateMatrix(height, width);
    (void) ReadIn(input_file_descriptor, input_data, input_data_matrix,
    NOVERTICALNOHORIZONTALFLIP, height, width, num_points);
    midgray = (NUMGRAYS / 2) - 1;
    histogram = (int *) SafeMalloc((unsigned) NUMGRAYS * sizeof(int),
    "histogram", "main");
    cumulative = (int *) SafeMalloc((unsigned) NUMGRAYS * sizeof(int),
    "cumulative", "main");
    desired = (int *) SafeMalloc((unsigned) NUMGRAYS * sizeof(int),
    "desired", "main");
    distribution = (int *) SafeMalloc((unsigned) NUMGRAYS * sizeof(int),
    "distribution", "main");
    clut =
    (unsigned char *) SafeMalloc((unsigned) NUMGRAYS * sizeof(unsigned char),
    "clut", "main");
    for(level = 0; level < NUMGRAYS; level++) {
	histogram[level] = 0;
	if(modification == EQUALIZATION) {
	    desired[level] = 1;
	} else if(modification == EXTREMES) {
	    desired[level] = (int) AbsInt(level - midgray);
	} else if(modification == MIDDLE) {
	    desired[level] = midgray - (int) AbsInt(level - midgray);
	}
    }
    for(sum = level = 0; level < NUMGRAYS; level++) {
	sum += desired[level];
    }
    for(level = 0; level < NUMGRAYS; level++) {
	desired[level] = (desired[level] * num_points) / sum;
    }
    for(row = 0; row < height; row++) {
	for(col = 0; col < width; col++) {
	    (histogram
	    [(int)(input_data_matrix[row][col]-1.0*MINGRAYVALUE+EPSILON)])++;
	}
    }
    cumulative[0] = histogram[0];
    distribution[0] = desired[0];
    for(level = 1; level < NUMGRAYS; level++) {
	cumulative[level] = cumulative[level-1] + histogram[level];
	distribution[level] = distribution[level-1] + desired[level];
    }
    for(fill = level = 0; level < NUMGRAYS; level++) {
	for(found = 0; (!found && (fill < NUMGRAYS));) {
	    if(cumulative[level] <= distribution[fill]) {
		found = 1;
	    } else {
		fill++;
	    }
	}
	clut[level] = (unsigned char) (MINGRAYVALUE + fill);
    }
    for(row = 0; row < height; row++) {
	for(col = 0; col < width; col++) {
	    output_data_matrix[row][col] =
	    clut[(int)(input_data_matrix[row][col]-1.0*MINGRAYVALUE+EPSILON)];
	}
    }
    (void) RealMatrixToRealVector(output_data, 
    output_data_matrix, height, width);
    (void) WriteOut(output_file_descriptor, NOVERTICALNOHORIZONTALFLIP, zoom, 
    output_data, height, width, num_points, COEFF);
    if((int) close(input_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, 
	"Error: Cannot close input file '%s'.  Exit 1.\n", input_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    if((int) close(output_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, 
	"Error: Cannot close output file '%s'.  Exit 1.\n",
	output_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) RealFreeVector(input_data);
    (void) RealFreeMatrix(input_data_matrix, height);
    (void) RealFreeVector(output_data);
    (void) RealFreeMatrix(output_data_matrix, height);
    (void) free(histogram);
    (void) free(cumulative);
    (void) free(desired);
    (void) free(distribution);
    (void) free(clut);
    (void) free(input_file_name);
    (void) free(output_file_name);
    (void) exit(0);
}
