#include "stdutils.h"
#include "complex.h"
#include "matrix.h"
#include "fft.h"
#include "inout.h"
#include "func.h"

void
main(argc, argv)
int argc;
char **argv;

{
    char *input_file_name = (char *) AllocateString(256);
    char *output_file_name = (char *) AllocateString(256);
    int input_file_descriptor;
    int output_file_descriptor;
    int height;
    int width;
    int power2_height;
    int power2_width;
    int zoom;
    int num_points;
    double *input_data;
    double **input_data_matrix;
    double *output_data;
    double **output_data_matrix;
    COMPLEX ***complex_input_data_matrix;
    COMPLEX ***complex_output_data_matrix;
    int row;
    int col;
    double mean;

    if(argc < 6) {
	(void) fprintf(stdout, 
	"Syntax: idft2d infile outfile height width zoom\n");
	(void) fflush(stdout);
	(void) exit(0);
    }
    (void) sscanf(argv[1], "%s", input_file_name);
    (void) sscanf(argv[2], "%s", output_file_name);
    (void) sscanf(argv[3], "%d", &height);
    (void) sscanf(argv[4], "%d", &width);
    (void) sscanf(argv[5], "%d", &zoom);
    input_file_descriptor = (int) open(input_file_name, O_RDONLY, 0);
    output_file_descriptor = (int) creat(output_file_name, 0644);
    if((num_points = (int) lseek(input_file_descriptor, 0, SEEK_END)) ==
    (int) NIL) {
	(void) fprintf(stdout, "Error: File %s nonexistent.  Exit 1.\n",
	input_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) lseek(input_file_descriptor, -(num_points), SEEK_END);
    for(power2_height = 1; power2_height < height; power2_height *= 2) {
	;
    }
    for(power2_width = 1; power2_width < width; power2_width *= 2) {
	;
    }
    input_data = (double *) RealCreateVector(num_points);
    input_data_matrix =
    (double **) RealCreateMatrix(power2_height, power2_width);
    complex_input_data_matrix = 
    (COMPLEX ***) ComplexCreateMatrix(power2_height, power2_width);
    output_data = (double *) RealCreateVector(num_points);
    output_data_matrix =
    (double **) RealCreateMatrix(power2_height, power2_width);
    complex_output_data_matrix = 
    (COMPLEX ***) ComplexCreateMatrix(power2_height, power2_width);
    (void) ReadIn(input_file_descriptor, input_data, input_data_matrix,
    NOVERTICALNOHORIZONTALFLIP, height, width, num_points);
    /*
    (void) RealScaleMatrix(input_data_matrix,
    input_data_matrix, 2.0 / (MAXGRAYVALUE - MINGRAYVALUE),
    power2_height, power2_width);
    for(row = 0; row < power2_height; row++) {
	for(col = 0; col < power2_width; col++) {
	    input_data_matrix[row][col] -= 1.0;
	}
    }
    */
    /*
    (void) RealAccumulateMatrix(&mean, input_data_matrix, height, width);
    mean /= num_points;
    for(row = 0; row < height; row++) {
	for(col = 0; col < width; col++) {
	    input_data_matrix[row][col] -= mean;
	}
    }
    */
    (void) RealMatrixToComplexMatrix(complex_input_data_matrix, 
    input_data_matrix, power2_height, power2_width);
    (void) IDFT2D(complex_output_data_matrix, 
    complex_input_data_matrix, power2_height, power2_width);
    (void) ComplexMatrixToRealVector(output_data, 
    complex_output_data_matrix, REALPART, height, width);
    (void) WriteOut(output_file_descriptor, NOVERTICALNOHORIZONTALFLIP, zoom, 
    output_data, height, width, num_points, COEFF);
    if((int) close(input_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, 
	"Error: Cannot close input file '%s'.  Exit 1.\n", input_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    if((int) close(output_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, 
	"Error: Cannot close output file '%s'.  Exit 1.\n", output_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) RealFreeVector(input_data);
    (void) RealFreeMatrix(input_data_matrix, power2_height);
    (void) ComplexFreeMatrix(complex_input_data_matrix,
    power2_height, power2_width);
    (void) RealFreeVector(output_data);
    (void) RealFreeMatrix(output_data_matrix, power2_height);
    (void) ComplexFreeMatrix(complex_output_data_matrix,
    power2_height, power2_width);
    (void) free(input_file_name);
    (void) free(output_file_name);
    (void) exit(0);
}
