#include "stdutils.h"
#include "complex.h"
#include "matrix.h"
#include "inout.h"
#include "func.h"

#define EXTENT 1
#define HMASK (2 * EXTENT + 1)
#define WMASK HMASK
#define MASKSIZE (HMASK * WMASK)

void main();
double Quantile();

void
main(argc, argv)
int argc;
char **argv;

{
    char *input_file_name = (char *) AllocateString(256);
    char *output_file_name = (char *) AllocateString(256);
    int input_file_descriptor;
    int output_file_descriptor;
    int height;
    int width;
    int zoom;
    int num_points;
    double *input_data;
    double **input_data_matrix;
    double *output_data;
    double **output_data_matrix;
    double *svec;
    int extent;
    int hmask;
    int wmask;
    int masksize;
    int row;
    int col;
    int n2;
    int n1;
    int index;
    double mean;

    if(argc < 6) {
	(void) fprintf(stdout, 
	"Syntax: median infile outfile height width zoom [extent]\n");
	(void) fprintf(stdout, 
	"Default extent is %d.\n",
	EXTENT);
	(void) fflush(stdout);
	(void) exit(0);
    }
    (void) sscanf(argv[1], "%s", input_file_name);
    (void) sscanf(argv[2], "%s", output_file_name);
    (void) sscanf(argv[3], "%d", &height);
    (void) sscanf(argv[4], "%d", &width);
    (void) sscanf(argv[5], "%d", &zoom);
    extent = EXTENT;
    if(argc > 6) {
	(void) sscanf(argv[6], "%d", &extent);
    }
    hmask = (2 * extent + 1);
    wmask = hmask;
    masksize = (hmask * wmask);
    (void) fprintf(stdout, 
    "Processing with an isotropic %d by %d median filter.\n",
    hmask, wmask);
    input_file_descriptor = (int) open(input_file_name, O_RDONLY, 0);
    output_file_descriptor = (int) creat(output_file_name, 0644);
    if((num_points = (int) lseek(input_file_descriptor, 0, SEEK_END)) ==
    (int) NIL) {
	(void) fprintf(stdout, "Error: File %s nonexistent.  Exit 1.\n",
	input_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) lseek(input_file_descriptor, -(num_points), SEEK_END);
    input_data = (double *) RealCreateVector(num_points);
    input_data_matrix = (double **) RealCreateMatrix(height, width);
    output_data = (double *) RealCreateVector(num_points);
    output_data_matrix = (double **) RealCreateMatrix(height, width);
    svec = (double *) RealCreateVector(masksize);
    (void) ReadIn(input_file_descriptor, input_data, input_data_matrix,
    NOVERTICALNOHORIZONTALFLIP, height, width, num_points);
    /*
    (void) RealScaleMatrix(input_data_matrix, 
    input_data_matrix, 2.0 / (MAXGRAYVALUE - MINGRAYVALUE), height, width);
    for(row = 0; row < height; row++) {
	for(col = 0; col < width; col++) {
	    input_data_matrix[row][col] -= 1.0;
	}
    }
    */
    (void) RealAccumulateMatrix(&mean, input_data_matrix, height, width);
    mean /= num_points;
printf("mean=%g\n",mean);fflush(stdout);
    /*
    for(row = 0; row < height; row++) {
	for(col = 0; col < width; col++) {
	    input_data_matrix[row][col] -= mean;
	}
    }
    */
    for(row = 0; row < height; row++) {
	for(col = 0; col < width; col++) {
	    for(n2 = -extent; n2 <= extent; n2++) {
		for(n1 = -extent; n1 <= extent; n1++) {
		    index = (n2 + extent) * hmask + (n1 + extent);
		    svec[index] = input_data_matrix
		    [(row-n2+height)%height][(col-n1+width)%width];
		}
	    }
	    output_data_matrix[row][col] =
	    (double) Quantile(svec, masksize / 2, masksize);
	}
    }
    (void) RealMatrixToRealVector(output_data, 
    output_data_matrix, height, width);
    (void) WriteOut(output_file_descriptor, NOVERTICALNOHORIZONTALFLIP, zoom, 
    output_data, height, width, num_points, SIGNAL);
    if((int) close(input_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, 
	"Error: Cannot close input file '%s'.  Exit 1.\n", input_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    if((int) close(output_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, 
	"Error: Cannot close output file '%s'.  Exit 1.\n",
	output_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) RealFreeVector(input_data);
    (void) RealFreeMatrix(input_data_matrix, height);
    (void) RealFreeVector(output_data);
    (void) RealFreeMatrix(output_data_matrix, height);
    (void) RealFreeVector(svec);
    (void) free(input_file_name);
    (void) free(output_file_name);
    (void) exit(0);
}

double
Quantile(vector, order, n)
double *vector;
int order;
int n;

{
    double *save_vector;
    double result;

    if((order >= 0) && (order < n)) {
	save_vector = (double *) RealCreateVector(n);
	(void) RealCopyVector(save_vector, vector, n);
	(void) RealHeapSortVector(save_vector, vector, n);
	result = save_vector[order];
	(void) RealFreeVector(save_vector);
	return(result);
    } else {
	(void) fprintf(stdout,
	"Error: Illegal quantile %d.  Exist 1.\n",
	order);
	(void) fflush(stdout);
	(void) exit(1);
	return((double) NIL);
    }
}
