
/*******************************************************************************************/
/*                        C ROUTINES FOR THE STEERABLE PYRMAID
                   COPYRIGHT 1993, PERCEPTUAL COMPUTING GROUP, MEDIA LABORATORY
		           MASSACHUSSETTS INSTITUTE OF TECHNOLOGY

*/


#include <stdio.h>
#include <math.h>
#include "basis.h"
#include <malloc.h>

/* routines to calculate dominant orientation  and magnitude */
void get_ori_g3h3();

void get_ori_g3h3(Ebasis,Obasis,x_comp,y_comp,xdim,ydim)
     Even_Basis Ebasis[5];
     Odd_Basis Obasis[4];
     float  **x_comp,**y_comp;
     int xdim,ydim;
{

  int i,j;
  float **a,**b;

  a=(float **)alloc_float_2D(xdim,ydim);
  b=(float **)alloc_float_2D(xdim,ydim);

/**************************************************************************/
/*                   COMPUTING X-COMPONENT                               */
/*************************************************************************/

  /* computing 0.375*g0^2 - 0.375*g2^2  */

  square(Obasis[0].pointer_basis,a,xdim,ydim);
  square(Obasis[2].pointer_basis,b,xdim,ydim);
  sub(a,b,a,xdim,ydim);
  mul_coeff(a,0.375,x_comp,xdim,ydim);
  /* computing + 0.17678(g0*g1 -g1*g2 - g0*g3 - g2*g3) */

  mul_im(Obasis[0].pointer_basis,Obasis[1].pointer_basis,a,xdim,ydim);
  mul_im(Obasis[1].pointer_basis,Obasis[2].pointer_basis,b,xdim,ydim);
  sub(a,b,a,xdim,ydim);

  mul_im(Obasis[0].pointer_basis,Obasis[3].pointer_basis,b,xdim,ydim);
  sub(a,b,a,xdim,ydim);
  mul_im(Obasis[2].pointer_basis,Obasis[3].pointer_basis,b,xdim,ydim);
  sub(a,b,a,xdim,ydim);
  mul_coeff(a,0.17678,a,xdim,ydim);
  add(x_comp,a,x_comp,xdim,ydim);

  /* computing + 0.25*g1*g3  */

  mul_im(Obasis[1].pointer_basis,Obasis[3].pointer_basis,a,xdim,ydim);
  mul_coeff(a,0.25,a,xdim,ydim);
  add(a,x_comp,x_comp,xdim,ydim);
  

  /* computing + 0.32*h0^2  */


  square(Ebasis[0].pointer_basis,a,xdim,ydim);
  mul_coeff(a,0.32,a,xdim,ydim);
  add(a,x_comp,x_comp,xdim,ydim);

  /* computing -0.16*h1*h4  */

  mul_im(Ebasis[1].pointer_basis,Ebasis[4].pointer_basis,a,xdim,ydim);
  mul_coeff(a,-0.16,a,xdim,ydim);
  add(a,x_comp,x_comp,xdim,ydim);

  /* computing + 0.098885*h1^2 + 0.098885*h4^2  */

  square(Ebasis[1].pointer_basis,a,xdim,ydim);
  square(Ebasis[4].pointer_basis,b,xdim,ydim);
  add(a,b,a,xdim,ydim);
  mul_coeff(a,0.098885,a,xdim,ydim);
  add(a,x_comp,x_comp,xdim,ydim);

  /* computing -0.0494422*(h0*h2 + h1*h2 + h3*h4 + h0*h3) */

  mul_im(Ebasis[0].pointer_basis,Ebasis[2].pointer_basis,a,xdim,ydim);
  mul_im(Ebasis[1].pointer_basis,Ebasis[2].pointer_basis,b,xdim,ydim);
  add(a,b,a,xdim,ydim);

  mul_im(Ebasis[3].pointer_basis,Ebasis[4].pointer_basis,b,xdim,ydim);
  add(a,b,a,xdim,ydim);
  mul_im(Ebasis[0].pointer_basis,Ebasis[3].pointer_basis,b,xdim,ydim);
  add(a,b,a,xdim,ydim);
  mul_coeff(a,-0.0494422,a,xdim,ydim);
  add(x_comp,a,x_comp,xdim,ydim);

  /* computing -0.159999*h2*h3   */

  mul_im(Ebasis[2].pointer_basis,Ebasis[3].pointer_basis,a,xdim,ydim);
  mul_coeff(a,-0.159999,a,xdim,ydim);
  add(a,x_comp,x_comp,xdim,ydim);

  /* computing -0.258885(h2^2 + h3^2) */

  square(Ebasis[2].pointer_basis,a,xdim,ydim);
  square(Ebasis[3].pointer_basis,b,xdim,ydim);
  add(a,b,a,xdim,ydim);
  mul_coeff(a,-0.258885,a,xdim,ydim);
  add(a,x_comp,x_comp,xdim,ydim);

  /* computing 0.129442*(h1*h3 + h0*h1 + h2*h4 + h0*h4) */
  

  mul_im(Ebasis[1].pointer_basis,Ebasis[3].pointer_basis,a,xdim,ydim);
  mul_im(Ebasis[0].pointer_basis,Ebasis[1].pointer_basis,b,xdim,ydim);
  add(a,b,a,xdim,ydim);

  mul_im(Ebasis[2].pointer_basis,Ebasis[4].pointer_basis,b,xdim,ydim);
  add(a,b,a,xdim,ydim);
  mul_im(Ebasis[0].pointer_basis,Ebasis[4].pointer_basis,b,xdim,ydim);
  add(a,b,a,xdim,ydim);
  mul_coeff(a,0.129442,a,xdim,ydim);
  add(x_comp,a,x_comp,xdim,ydim);
  
/**********************************************************************/
/*                      COMPUTING Y-COMPONENT                         */
/**********************************************************************/


  /* computing 0.375*g1^2 - 0.375*g3^2  */

  square(Obasis[1].pointer_basis,a,xdim,ydim);
  square(Obasis[3].pointer_basis,b,xdim,ydim);
  sub(a,b,a,xdim,ydim);
  mul_coeff(a,0.375,y_comp,xdim,ydim);

  /* computing + 0.17678(g0*g1 +g1*g2 + g0*g3 - g2*g3) */

  mul_im(Obasis[0].pointer_basis,Obasis[1].pointer_basis,a,xdim,ydim);
  mul_im(Obasis[1].pointer_basis,Obasis[2].pointer_basis,b,xdim,ydim);
  add(a,b,a,xdim,ydim);

  mul_im(Obasis[0].pointer_basis,Obasis[3].pointer_basis,b,xdim,ydim);
  add(a,b,a,xdim,ydim);
  mul_im(Obasis[2].pointer_basis,Obasis[3].pointer_basis,b,xdim,ydim);
  sub(a,b,a,xdim,ydim);
  mul_coeff(a,0.17678,a,xdim,ydim);
  add(y_comp,a,y_comp,xdim,ydim);

  /* computing - 0.25*g0*g2  */

  mul_im(Obasis[0].pointer_basis,Obasis[2].pointer_basis,a,xdim,ydim);
  mul_coeff(a,-0.25,a,xdim,ydim);
  add(y_comp,a,y_comp,xdim,ydim);
  

  /* computing + 0.304338*h1^2 - 0.304338*h4^2  */

  square(Ebasis[1].pointer_basis,a,xdim,ydim);
  square(Ebasis[4].pointer_basis,b,xdim,ydim);
  sub(a,b,a,xdim,ydim);
  mul_coeff(a,0.304338,a,xdim,ydim);
  add(a,y_comp,y_comp,xdim,ydim);

  /* computing +0.152169*(-h0*h2 + h1*h2 - h3*h4 + h0*h3) */

  mul_im(Ebasis[0].pointer_basis,Ebasis[2].pointer_basis,a,xdim,ydim);
  mul_im(Ebasis[1].pointer_basis,Ebasis[2].pointer_basis,b,xdim,ydim);
  sub(b,a,a,xdim,ydim);

  mul_im(Ebasis[3].pointer_basis,Ebasis[4].pointer_basis,b,xdim,ydim);
  sub(a,b,a,xdim,ydim);
  mul_im(Ebasis[0].pointer_basis,Ebasis[3].pointer_basis,b,xdim,ydim);
  add(a,b,a,xdim,ydim);
  mul_coeff(a,0.152169,a,xdim,ydim);
  add(y_comp,a,y_comp,xdim,ydim);

  /* computing 0.188091(h2^2 - h3^2) */

  square(Ebasis[2].pointer_basis,a,xdim,ydim);
  square(Ebasis[3].pointer_basis,b,xdim,ydim);
  sub(a,b,a,xdim,ydim);
  mul_coeff(a,0.188091,a,xdim,ydim);
  add(a,y_comp,y_comp,xdim,ydim);

  /* computing 0.094045*(h0*h1 - h3*h1 + h2*h4 - h0*h4) */
  

  mul_im(Ebasis[1].pointer_basis,Ebasis[3].pointer_basis,b,xdim,ydim);
  mul_im(Ebasis[0].pointer_basis,Ebasis[1].pointer_basis,a,xdim,ydim);
  sub(a,b,a,xdim,ydim);

  mul_im(Ebasis[2].pointer_basis,Ebasis[4].pointer_basis,b,xdim,ydim);
  add(a,b,a,xdim,ydim);
  mul_im(Ebasis[0].pointer_basis,Ebasis[4].pointer_basis,b,xdim,ydim);
  sub(a,b,a,xdim,ydim);
  mul_coeff(a,0.094045,a,xdim,ydim);
  add(y_comp,a,y_comp,xdim,ydim);
  
  f_free2d(a,ydim);
  f_free2d(b,ydim);


}







