/*                        C ROUTINES FOR THE STEERABLE PYRMAID
                   COPYRIGHT 1993, PERCEPTUAL COMPUTING GROUP, MEDIA LABORATORY
		           MASSACHUSSETTS INSTITUTE OF TECHNOLOGY
                                      MONIKA GORKANI
*/


#include <stdio.h>
#include <math.h>
#include <dat.h>
#include <malloc.h>


float **read_image();            /* function to read image */
char *alloc_char();            /* function to allocate memory for type unsigned char  */
float *alloc_float();           /* function to allocate memory for type float */
void cstream_array_2D();     /* convert 1D array of type unsigned char to 2D float array*/
void fstream_array_2D();     /* convert 1D array of type float to 2D float array */
void array_2D_cstream();     /* convert 2D array of type float to 1D character array */
void array_2D_fstream();     /* convert 2D array of type float to 1D float array */
void write_image();        /* function to write image */
float **alloc_float_2D();     /* allocate memory for 2D array of type float */
unsigned char *alloc_uchar();  /* allocate memory for 2D array of type unsigned character */
void f_free2d() ;            /* free memory for 2D array of type float */

float **read_image(xdim,ydim,filename,type)
     char *filename;
     int *xdim, *ydim,type;
{
  float **im1;
  unsigned char *inputDatac;
  float *inputDataf;
  DATFILE *inputDat;
  int xy[2],n,imsize,i,j;
  int imageDatType=0;
  int bps=0;

/* opening image file */
 
  if ((inputDat = d_open(filename, "r")) == (DATFILE*)NULL)
    {
      fprintf(stderr, "Can't open filter file\n");
      exit(1);
    }
 
/* find dimension of input image and type of image  */

  d_getdim(inputDat, &n, &n, xy, 2);
  *xdim=xy[1];
  *ydim=xy[0];

/* used to be xdim=xy[1] and ydim=xy[0] */

  im1=(float **)alloc_float_2D(*xdim,*ydim);
  

  if (type == 'b') {
    imsize= *xdim * *ydim;
       
    inputDatac=(unsigned char*)alloc_uchar(imsize);
    
    if ((i = read(fileno(d_getfp(inputDat)), inputDatac, imsize)) != imsize)
      {
	fprintf(stderr, "File reading error - insufficient samples\n");
	exit(1);
      }

    
    cstream_array_2D(inputDatac,im1,*xdim,*ydim);
    free(inputDatac);
    return(im1);
}

  else if (imageDatType = 'f') {
    imsize= *xdim * *ydim * sizeof(float);
    
    inputDataf=alloc_float(imsize);
   
    if ((i = read(fileno(d_getfp(inputDat)), inputDataf, imsize)) != imsize)
      {
	fprintf(stderr, "File reading error - insufficient samples\n");
	exit(1);
      }
 
   
   
   
    fstream_array_2D(inputDataf,im1,*xdim,*ydim);
    
    free(inputDataf);
    return(im1);
  }

  else {
    fprintf(stderr, "Image Type not float or unsigned char \n");
    exit(1);
  }
}


void write_image(im1,xdim,ydim,filename,type)
     float **im1;
     char *filename;
     int xdim, ydim;
     int type;
{
  unsigned char *outputDatac;
  float *outputDataf;
  DATFILE *outDat;
  int xy[2],n,imsize,i;
  int imageDatType=0;
  int nbps;

/* opening image file */

  if ((outDat = d_open(filename, "w")) == (DATFILE*)  NULL)
    {
      fprintf(stderr, "Can't open filter file\n");
      exit(1);
    }
 
/* create descriptor for output image  */

  xy[1]=xdim;
  xy[0]=ydim;

/* used to be xy[0]= ydim and xy[1]=xdim */
  d_setdim(outDat,1,1,xy,2);
 
 
/* if type=b then write as unsigned char else type =f then as float */

  if (type =='b') {
    imsize=xdim*ydim;
    d_setkey(outDat, TYPE, UNSIGNED1);
    
    outputDatac=(unsigned char *)alloc_uchar(imsize);
    array_2D_cstream(outputDatac,im1,xdim,ydim);
    if (write(fileno(d_getfp(outDat)), outputDatac, imsize) != imsize)
    {
      printf("Unable to write to output file.\n");
      exit(1);
      
    }
  
    free(outputDatac);
  }

  else if (type=='f') {

    imsize=xdim*ydim*sizeof(float);
    d_setkey(outDat, TYPE, FLOAT);
   
    outputDataf=alloc_float(imsize);
   
    array_2D_fstream(outputDataf,im1,xdim,ydim);
    
    if (write(fileno(d_getfp(outDat)), outputDataf, imsize) != imsize)
    {
      printf("Unable to write to output file.\n");
      exit(1);
      
    }
    
    free(outputDataf);

 }
  
  else {
    fprintf(stderr, "Image Type not float or unsigned char \n");
    exit(1);
  }
}


char *alloc_char(size)
    
     int size;
{
  char *ptr;
   if ((ptr = (char*) malloc(size)) == NULL)
     {
       fprintf(stderr, "Can't allocate memory for image\n");
       exit(1);
     }
   return(ptr);
 }
 
unsigned char *alloc_uchar(size)
    
     int size;
{
  unsigned char *ptr;
   if ((ptr = (unsigned char*) malloc(size)) == NULL)
     {
       fprintf(stderr, "Can't allocate memory for image\n");
       exit(1);
     }
   return(ptr);
 }
 




float *alloc_float(size)
    
     int size;
{
  float *ptr;
  int i;
  
  if ((ptr = (float *) malloc(size)) == NULL)
    {
      fprintf(stderr, "Can't allocate memory for image\n");
      exit(1);
    }
  return(ptr);
}

float **alloc_float_2D(xdim,ydim)
     int xdim,ydim;
{
  float **im1;
  int i,j;
  if ((im1 = (float **) malloc((unsigned) ydim *sizeof(float *))) == NULL) {
    printf("Can't allocate memory \n");
    exit(0);
  }

   for (i=0; i<ydim; i++)
    if ((im1[i]= (float *) calloc((unsigned) xdim, sizeof(float))) == NULL)
      {
	fprintf(stderr, "Can't allocate memory for filter data\n");
	exit(1);
      }
  return(im1);
}



void cstream_array_2D(cptr,fptr,xdim,ydim)
     unsigned char *cptr;
     float **fptr;
     int xdim,ydim;
{
  int i,j;

  for (i=0;i<ydim;i++)
    {
      for (j=0;j<xdim;j++)
	{
	  fptr[i][j]= (float) *(cptr + j + i*xdim);
	}
    }

}

void fstream_array_2D(ptr,fptr,xdim,ydim)
     float *ptr;
     float **fptr;
     int xdim,ydim;
{
  int i,j;


  for (i=0;i<ydim;i++)
    {
      for (j=0;j<xdim;j++)
	{
	 
	  fptr[i][j]= *(ptr + j + i*xdim);
	 
	}
    }
 
}


void array_2D_cstream(cptr,im1,xdim,ydim)
     unsigned char *cptr;
     float **im1;
     int xdim,ydim;

{
  int i,j;
  float max,min,diff;
  max=im1[0][0];
  min=im1[0][0];

  for (i=0;i<ydim;i++)
    {
      for (j=0;j<xdim;j++)
	{
	  if (im1[i][j]> max)
	    max=im1[i][j];
	  if (im1[i][j]< min)
	    min=im1[i][j];
	}
    }
  diff=max-min;

  for (i=0;i<ydim;i++)
    {
      for (j=0;j<xdim;j++)
	{
	  *(cptr + j + i*xdim) = (unsigned char) (255 - (max - im1[i][j])/diff*255);
	}
    }

}


void array_2D_fstream(fptr,im1,xdim,ydim)
     float *fptr;
     float **im1;
     int xdim,ydim;
{
  int i,j;
  for (i=0;i<ydim;i++)
    {
      for (j=0;j<xdim;j++)
	{
	  *(fptr + j + i*xdim) = im1[i][j];
	}
    }

}

void f_free2d (m,ydim)
     float   **m;
     int     ydim;
{
  
  int   i;
  
  for (i=0; i<ydim; i++)
    free((char *) m[i]);
  free((char *) m);
}









