#include <stdio.h>
#include <math.h>
#include "basis.h"  /* description of data structures used */
#include "m.h"      /* description of math utilities */
#include "io.h"     /* description of input and output and memory 
                      allocation utilites */
#include "pyr_func.h"  /* description of steerable pyramid functions */



main(argc,argv)

     int argc;
     char *argv[];

{


/* your can alter the arguments argv  to see various things like the basis 
filter outputs, the dominant orientation and magnitude, the low-pass 
down-sampled filter images , the steered output of an image, etc... */

/* for this demonstration we'll output the dominant orientation image and
its corresponding strength image 

*/

  if (argc != 5)
    {
      fprintf(stderr, "steer_pyr image_name pyramid_level name_for_angle_image name_for_strength_image");
      exit(-1);
      
    }

/***********************************************************************
 
                         DECLARATION OF VARIABLES 

***********************************************************************/

  int level;   /* the desired pyramid level */
  float **im1;  /* pointer to input image (the images are treated as 2D arrays*/
  char type;  /* type of image (ie. unsigned char or float) */
  int xdim,ydim;  /* dimensions of the input image */

  Even_Filter Efilter[5]; /* array of even filter structure */
  Odd_Filter Ofilter[4];  /* array of odd filter structure */
  Even_Basis Ebasis[5]; /* array of even basis outputs */
  Odd_Basis Obasis[4];  /* array of odd basis outputs */
  float **tmp;  /* pointer to temp file */
  int xfdim,yfdim; /* dimensions of the basis filters */
  char *filename;  /* holds the names of filters */
  float **angle,**mag;  /* images corresponding to the dominant direction 
                        and its strength */
  float **energy;  /* image corresponding to the directional energy */
  float **steer_image;  /* image corresponding to the steered image */
  float direction;  /* direction to steer filter */



/************************************************************************/




  filename=(char *)alloc_char(100);
  

    
  /* Desired  Pyramid Level  */

  level=atol(argv[2]);
  
  if (level <0 )
    {
      fprintf(stderr, "The Pyramid level you want is less than zero!!! \n");
      exit(-1);
    }


  /* Reading the input image (assuming the image to be a datfile )*/

  type='b';   /* assuming the input image is unsigned char */

  im1=(float**)read_image(&xdim,&ydim,argv[1],type); 


  /* Access the low-pass down-sampled version of input image corresponding
     to the desired pyramid level 
  */

  access_low(im1,&xdim,&ydim,level);

  /* im1 will now point to the low-pass down-sampled image and xdim and ydim will be the new (smaller) dimensions */


/*************************************************************************/
/*****           Finding the pyramid basis outputs             ***********/
/*************************************************************************/

  tmp=(float **)alloc_float_2D(xdim,ydim);
 


/*********************************************************************/
/*                   Creating Odd filters                            */
/*********************************************************************/
    

  filename="~freeman/lisp/steer/bpoddc0";
  Ofilter[0].pointer_filter=(float **)read_image(&xfdim,&yfdim,filename,'f');
  filename="~freeman/lisp/steer/bpoddc1";
  Ofilter[1].pointer_filter=(float **)read_image(&xfdim,&yfdim,filename,'f');
  filename="~freeman/lisp/steer/bpoddc2";
  Ofilter[2].pointer_filter=(float **)read_image(&xfdim,&yfdim,filename,'f');
  filename="~freeman/lisp/steer/bpoddc3";
  Ofilter[3].pointer_filter=(float **)read_image(&xfdim,&yfdim,filename,'f');
 

  for (i=0;i<4;i++) {
    Ofilter[i].xdim=xfdim;
    Ofilter[i].ydim=yfdim;
  }

/*********************************************************************/
/*                  Creating Even filters                            */
/*********************************************************************/

  
  filename="~freeman/lisp/steer/bpevenb0";
  Efilter[0].pointer_filter=(float **)read_image(&xfdim,&yfdim,filename,'f');
  filename="~freeman/lisp/steer/bpevenb1";
  Efilter[1].pointer_filter=(float **)read_image(&xfdim,&yfdim,filename,'f');
  filename="~freeman/lisp/steer/bpevenb2";
  Efilter[2].pointer_filter=(float **)read_image(&xfdim,&yfdim,filename,'f');
  filename="~freeman/lisp/steer/bpevenb3";
  Efilter[3].pointer_filter=(float **)read_image(&xfdim,&yfdim,filename,'f');
  filename="~freeman/lisp/steer/bpevenb4";
  Efilter[4].pointer_filter=(float **)read_image(&xfdim,&yfdim,filename,'f');
 

  for (i=0;i<5;i++) {
    Efilter[i].xdim=xfdim;
    Efilter[i].ydim=yfdim;
  }

/*********************************************************************/
/*                allocating memory for odd and even basis outputs   */
/*********************************************************************/

  
  Obasis[0].pointer_basis=(float **)alloc_float_2D(xdim,ydim);
  Obasis[1].pointer_basis=(float **)alloc_float_2D(xdim,ydim);
  Obasis[2].pointer_basis=(float **)alloc_float_2D(xdim,ydim);
  Obasis[3].pointer_basis=(float **)alloc_float_2D(xdim,ydim);



  Ebasis[0].pointer_basis=(float **)alloc_float_2D(xdim,ydim);
  Ebasis[1].pointer_basis=(float **)alloc_float_2D(xdim,ydim);
  Ebasis[2].pointer_basis=(float **)alloc_float_2D(xdim,ydim);
  Ebasis[3].pointer_basis=(float **)alloc_float_2D(xdim,ydim);
  Ebasis[4].pointer_basis=(float **)alloc_float_2D(xdim,ydim);
 
  
  

/*********************************************************************/
/*                   Make quadrature basis outputs                   */
/*********************************************************************/

  make_quad_basis_ns(im1,tmp,Efilter,Ofilter,Ebasis,Obasis,5,4,xdim,ydim);
 
  free(tmp);



/*********************************************************************/
/*         GET DOMINANT DIRECTION AND ITS CORRESPONDING STRENGTH 
           FOR EACH PIXEL POSITION AT THE DESIRED PYRAMID LEVEL
*********************************************************************/
  angle=(float **)alloc_2D_float(xdim,ydim);
  mag=(float **)alloc_2D_float(xdim,ydim);

  get_angle_mag(Ebasis,Obasis,angle,mag,xdim,ydim);


/*********************************************************************
/*                Get the steered filtered output
*********************************************************************/
  steer_image=(float **)alloc_2D_float(xdim,ydim);

/* Lets steer the image using the odd basis filters. Alternatively can 
   use even basis filters by calling steer_even()                    */

  direction=M_PI;  /* Just for fun choose Pi */

  steer_odd(Obasis,direction,xdim,ydim,steer_image);


/*********************************************************************
/*                Get the directional energy output
*********************************************************************/
	    
  energy=(float **)alloc_2D_float(xdim,ydim);
  dir_energy(Ebasis,Obasis,direction,xdim,ydim,energy);



/**********************************************************************
/* Finally output the dominant orientation and its strength   */

  /* write angle image */
  write_image(angle,xdim,ydim,argv[3],'b');
  
  /* write mag image */

  write_image(mag,xdim,ydim,argv[4], 'b');




}

