/*                        C ROUTINES FOR THE STEERABLE PYRMAID
                   COPYRIGHT 1993, PERCEPTUAL COMPUTING GROUP, MEDIA LABORATORY
		           MASSACHUSSETTS INSTITUTE OF TECHNOLOGY

*/

#include <stdio.h>
#include <math.h>
#include <malloc.h>


void square(inptr,outptr,xdim,ydim)
     register float **inptr,**outptr;
     register int xdim,ydim;
{

  register int i,j;
  for (i=0;i<ydim;i++)
    {
      for (j=0;j<xdim;j++)
	{
	  outptr[i][j]=inptr[i][j]*inptr[i][j];
	}
    }

}
void square_root(inptr,outptr,xdim,ydim)
     float **inptr,**outptr;
     register int xdim,ydim;
{

  register int i,j;
  for (i=0;i<ydim;i++)
    {
      for (j=0;j<xdim;j++)
	{
	  outptr[i][j]=sqrt(inptr[i][j]);
	  printf("%f \n",outptr[i][j]);
	}
    }

}


void mul_im(inptr1,inptr2,outptr,xdim,ydim)
     register float **inptr1,**outptr,**inptr2;
     register int xdim,ydim;
{

  register int i,j;
  for (i=0;i<ydim;i++)
    {
      for (j=0;j<xdim;j++)
	{
	  outptr[i][j]=inptr1[i][j]*inptr2[i][j];
	}
    }

}


void mul_coeff(inptr,coeff,outptr,xdim,ydim)
     register float **inptr,coeff,**outptr;
     register int xdim,ydim;
{

  register int i,j;
  for (i=0;i<ydim;i++)
    {
      for (j=0;j<xdim;j++)
	{
	  outptr[i][j]=inptr[i][j]*coeff;
	}
    }

}

     

void add (inptr1,inptr2,outptr,xdim,ydim)
     register float **inptr1,**outptr,**inptr2;
     register int xdim,ydim;
{

  register int i,j;
  for (i=0;i<ydim;i++)
    {
      for (j=0;j<xdim;j++)
	{
	  outptr[i][j]=inptr1[i][j]+inptr2[i][j];
	}
    }

}


void sub (inptr1,inptr2,outptr,xdim,ydim)
     register float **inptr1,**outptr,**inptr2;
     register int xdim,ydim;
{

  register int i,j;
  for (i=0;i<ydim;i++)
    {
      for (j=0;j<xdim;j++)
	{
	  outptr[i][j]=inptr1[i][j] - inptr2[i][j];
	}
    }

}


void complex_phase(im1, im2, result, xdim,ydim)
     register float **im1, **im2, **result;
     register int xdim,ydim;
{  
  register int i,j;
  register double im2val, im1val;
  
  for (i=0; i<ydim; i++)
    {
      for (j=0; j<xdim; j++)
	{ 

      im1val = im1[i][j];  im2val = im2[i][j];
      if (im1val == 0.0)
	{
	  if (im2val == 0.0) result[i][j] = 0.0;
	  else if (im2val < 0.0) result[i][j] = -M_PI/2.0;
	  else result[i][j] = M_PI/2.0;
	}
      else if (im1val < 0.0)
	{
	  if (im2val == 0.0) result[i][j] = - M_PI;
	  if (im2val < 0.0) result[i][j] = atan(im2val / im1val) - M_PI;
	  else result[i][j] = atan(im2val / im1val) + M_PI;
	}
      else result[i][j] = atan(im2val / im1val);
    }
    }
}

void magnitude (im1, im2, result, xdim, ydim)
     register float **im1, **im2, **result;
     register int xdim,ydim;
     
  {
    register int i,j;
    
    for (i=0; i<ydim; i++)
      {
	for (j=0; j<xdim; j++)
	  {
	    result[i][j]= (float)sqrt((double)(im1[i][j]*im1[i][j] + im2[i][j]*im2[i][j]));
	  }
      }

  }


void pow_image(inptr1,num,outptr,xdim,ydim)
     register float **inptr1,**outptr;
     register int xdim,ydim;
     float num;
{

  register int i,j;
  for (i=0;i<ydim;i++)
    {
      for (j=0;j<xdim;j++)
	{
	  outptr[i][j]=pow(inptr1[i][j],num);
	}
    }

}

void cos_image(inptr1,outptr,xdim,ydim)
     register float **inptr1,**outptr;
     register int xdim,ydim;
{

  register int i,j;
  for (i=0;i<ydim;i++)
    {
      for (j=0;j<xdim;j++)
	{
	  outptr[i][j]=cos(inptr1[i][j]);
	}
    }

}

void sin_image(inptr1,outptr,xdim,ydim)
     register float **inptr1,**outptr;
     register int xdim,ydim;
{

  register int i,j;
  for (i=0;i<ydim;i++)
    {
      for (j=0;j<xdim;j++)
	{
	  outptr[i][j]=sin(inptr1[i][j]);
	}
    }

}


