
/*******************************************************************************************/
/*                        C ROUTINES FOR THE STEERABLE PYRMAID
                   COPYRIGHT 1993, PERCEPTUAL COMPUTING GROUP, MEDIA LABORATORY
		           MASSACHUSSETTS INSTITUTE OF TECHNOLOGY

*/


#include "basis.h"
#include <stdio.h>
#include <math.h>
#include "io.h"
#include "m.h"
#include "pyr_func.h"


void access_low(im1,xdim,ydim,nlevel)
     float **im1;
     int *xdim,*ydim;
     int nlevel;

{

  char *filename; /* pointer to the name of the low-pass filter */
  Low_Filter Lfilt; /*structure corresponding to the low-pass filter */
  int xfdim,yfdim ; /* dimensions of the low-pass filter */
  int i;

  filename=(char *)alloc_char(100);
  filename="/v/users/freeman/lisp/steer/lpfb";

  /* assigning memory for the low-pass filter */
  Lfilt.pointer_filter=(float**)read_image(&xfdim,&yfdim,filename,'f'); 
  
  /* assigning the dimensions of the filter */
  Lfilt.xdim=xfdim;
  Lfilt.ydim=yfdim;
  for (i=0;i<=nlevel;i++)
    {
      /* make low-pass version of image and subsample */
      make_low_image(im1,Lfilt,i,xdim,ydim);
      
    }

}


void make_low_image(im1,Lfilt,level,xdim,ydim)
     float **im1;
     Low_Filter Lfilt;
     int level,*xdim,*ydim;
{
  int i,j,k1,k2;
  float **tmp1,**tmp2;
  
  tmp1=(float **)alloc_float_2D(*xdim,*ydim);
  tmp2=(float **)alloc_float_2D(*xdim,*ydim);
  
  if (level>0)
    {
       
      twod_conv_ns(im1,tmp1,tmp2,Lfilt.pointer_filter,*xdim,*ydim,Lfilt.xdim,Lfilt.ydim,1,0);
      downsample2d(tmp1,im1,*xdim,*ydim,2,2);
      
      *xdim = *xdim/2; 
      *ydim = *ydim/2;
    }

  f_free2d(tmp1,*ydim);
  f_free2d(tmp2,*ydim);
 
 
}


void make_quad_basis_ns (im1,tmp,Efilt,Ofilt,Ebasis,Obasis,Enum,Onum,xdim,ydim)

     float **im1;   /* input image  */
     float **tmp;   /* temp image */
     Even_Filter Efilt[];  /* array of structures for even filter */
     Odd_Filter Ofilt[];   /* array of structures for odd filter */
     Even_Basis Ebasis[];   /* array of structures for even basis  */
     Odd_Basis Obasis[];  /* array of structures for odd basis */ 
     int Enum,Onum,xdim,ydim; /* number of basis filters for odd and even filters */

{
  int i,j ;
  
  for (i=0;i<Enum;i++)
    {
     
      twod_conv_ns(im1,Ebasis[i].pointer_basis,tmp,Efilt[i].pointer_filter,xdim,ydim,Efilt[i].xdim,Efilt[i].ydim,1,0);
     
    }
  
  for (i=0; i<Onum; i++)
    {
      twod_conv_ns(im1,Obasis[i].pointer_basis,tmp,Ofilt[i].pointer_filter,xdim,ydim,Ofilt[i].xdim,Ofilt[i].ydim,1,0);
    }

  }



void get_angle_mag(Ebasis,Obasis,angle,mag,xdim,ydim)
     Even_Basis Ebasis[];
     Odd_Basis Obasis[];
     float **angle,**mag;
     int xdim,ydim;

{

  float **x_comp,**y_comp;



/**************************************************************************/
/* CALCULATE THE X AND Y COMPONENTS FOR FINDING DOMINANT ORIENTATION  */
/**************************************************************************/


  x_comp=(float **)alloc_float_2D(xdim,ydim);
  y_comp=(float **)alloc_float_2D(xdim,ydim);

  get_ori_g3h3(Ebasis,Obasis,x_comp,y_comp,xdim,ydim);



/*********************************************************************/
/*              CALCULATE THE DOMINANT ORIENTATION IMAGE             */
/*********************************************************************/

  /* Returns magnitude of the dominant orientation at each pixel position */

  magnitude(x_comp,y_comp,mag,xdim,ydim); 
  
  /* Returns double of the dominant orientation */
  complex_phase(x_comp,y_comp,angle,xdim,ydim);

  /* Returns the dominant orientation at each pixel position in degrees*/
  mul_coeff(angle,0.5,angle,xdim,ydim);

  f_free2d(x_comp,ydim);
  f_free2d(y_comp,ydim);

  }


void steer(Obasis,angle,xdim,ydim,result)
     float angle;
     Odd_Basis Obasis[];
     float **result;
     int xdim,ydim;

{

  int i,j;
  float a,b,c,d;
  float **temp;
  
  temp=(float **)alloc_float_2D(xdim,ydim);
  
  a= cos(angle);
  c= 2*pow(a,3.0) - a;
  
  mul_coeff(Obasis[0].pointer_basis,c,result,xdim,ydim);

  c=cos(angle - (M_PI/4));
  d=2*pow(c,3.0) - c;
  mul_coeff(Obasis[1].pointer_basis,d,temp,xdim,ydim);
  add(result,temp,result,xdim,ydim);

  c=cos(angle - (M_PI/2));
  d=2*pow(c,3.0)-c;
  mul_coeff(Obasis[2].pointer_basis,d,temp,xdim,ydim);
  add(result,temp,result,xdim,ydim);

  c=cos(angle - (3*M_PI/4));
  d=2*pow(c,3.0)-c;
  mul_coeff(Obasis[3].pointer_basis,d,temp,xdim,ydim);
  add(result,temp,result,xdim,ydim);
  f_free2d(temp,ydim);

}

void steer_image(Obasis,angle_image,xdim,ydim,result)
     float **angle_image;
     Odd_Basis Obasis[];
     float **result;
     int xdim,ydim;

{


  float **a,**b,**c,**d;

  
  c=(float **)alloc_float_2D(xdim,ydim);
  a=(float **)alloc_float_2D(xdim,ydim);
  b=(float **)alloc_float_2D(xdim,ydim);
  d=(float **)alloc_float_2D(xdim,ydim);

  sin_image(angle_image,a,xdim,ydim);
  cos_image(angle_image,b,xdim,ydim);

  pow_image(b,3.0,c,xdim,ydim); /* cos^3 */
  mul_coeff(c,2.0,c,xdim,ydim);
  sub(c,b,c,xdim,ydim); /* coefficient for first basis output */
  mul_im(Obasis[0].pointer_basis,c,result,xdim,ydim);
  
  add(a,b,c,xdim,ydim);
  mul_coeff(c,0.7071,c,xdim,ydim); /* 0.7071*(cos + sin) */
  pow_image(c,3.0,d,xdim,ydim);
  mul_coeff(d,2.0,d,xdim,ydim);
  sub(d,c,d,xdim,ydim);  /* coefficient for 2nd basis output */
  mul_im(Obasis[1].pointer_basis,d,c,xdim,ydim);
  add(result,c,result,xdim,ydim);

  pow_image(a,3.0,c,xdim,ydim);  /* sin^3 */
  mul_coeff(c,2.0,c,xdim,ydim);
  sub(c,a,c,xdim,ydim); /* coefficient for 3rd basis output */
  mul_im(Obasis[2].pointer_basis,c,d,xdim,ydim);
  add(result,d,result,xdim,ydim);

  sub(a,b,c,xdim,ydim);  
  mul_coeff(c,0.7071,c,xdim,ydim); /* 0.7071 * (sin - cos) */
  pow_image(c,3.0,d,xdim,ydim);
  mul_coeff(d,2.0,d,xdim,ydim);
  sub(d,c,d,xdim,ydim); /* coefficient for 3rd basis output */
  mul_im(Obasis[3].pointer_basis,d,c,xdim,ydim);
  add(result,c,result,xdim,ydim);
  
  f_free2d(a,ydim);
  f_free2d(b,ydim);
  f_free2d(c,ydim);
  f_free2d(d,ydim);

}


  








