
/*******************************************************************************************/
/*                        C ROUTINES FOR THE STEERABLE PYRMAID
                   COPYRIGHT 1993, PERCEPTUAL COMPUTING GROUP, MEDIA LABORATORY
		           MASSACHUSSETTS INSTITUTE OF TECHNOLOGY

*/

/**************************************************************************/

/*            STEERABLE PYRAMID FUNCTION DESCRIPTIONS                     */

/**************************************************************************/






/* access_low(im1,xdim,ydim,nlevel) -  gives the low-pass downsampled image 
                                      corresponding to the nlevel of pyramid */


/*  float  **im1 =  pointer to the image  
    int  *xdim,*ydim = dimensions of the image
                  routines returns xdim/2^nlevel ydim/2^nlevel dimensions 
                  corresponding to the down-sampled image

    int  nlevel = nth level of the pyramid   

*/


/* make_low_image(im1,Lfilt,level,xdim,ydim) - the routine called by
  access_low to convolve the image with a low-pass filter and downsample 

*/


/* make_quad_basis_ns(im1,tmp,Efilt,Ofilt,Ebasis,Obasis,Enum,Onum,xdim,ydim) 
   Routine to make the even and odd basis outputs using non-separable filters 

  float **im1   input image  
  float **tmp  temp image 
  Even_Filter Efilt[]  array of structures for even filter 
  Odd_Filter Ofilt[]    array of structures for odd filter 
  Even_Basis Ebasis[]   array of structures for even basis  
  Odd_Basis Obasis[]  array of structures for odd basis  
  int Enum,Onum,xdim,ydim  number of basis filters for odd and even filters 


*/


/* get_angle_mag(Ebasis,Obasis,angle,mag,xdim,ydim) - returns angle and 
   mag images corresponding to the dominant orientation and its corresponding
   strength for each pixel position   

   Even_basis Ebasis - structure corresponding to even basis outputs
   Odd_basis Obasis - structure corresponding to odd basis outputs
   angle - angle image corresponding to the dominant orientations at each pixel 
           position (will be degrees)

   mag -  strength image correponding to the strengths of dominant orientations
          at each position  

*/

/*  steer(Obasis,angle,xdim,ydim,result) - routine which runs the 
    steerable pyramid filter steered to a certain orientation 
    using the four basis outputs.
    Odd_basis Obasis - structure corresponding to odd basis outputs
    float angle - angle to steer to (just a number) (in radians)
    int xdim,ydim - dimensions of the basis outputs
    float **result - the steered output
    
*/   


/* steer_image(Obasis,angle,xdim,ydim,result) - routine to steer however 
   it inputs an angle image rather than just one number. This way at each 
   point of the image the output is steered to a particular direction */


void steer();
void steer_image();
void make_low_image();
void access_low();
void make_quad_basis_ns();
void get_angle_mag();




