#include "stdutils.h"
#include "complex.h"
#include "matrix.h"
#include "inout.h"
#include "func.h"

#define NUMGRAYS (MAXGRAYVALUE - MINGRAYVALUE + 1)
#define HEIGHT NUMGRAYS
#define BINSIZE 2
#define WIDTH (BINSIZE * HEIGHT)

void
main(argc, argv)
int argc;
char **argv;

{
    char *input_file_name = (char *) AllocateString(256);
    char *hist_file_name = (char *) AllocateString(256);
    char *cum_file_name = (char *) AllocateString(256);
    int input_file_descriptor;
    int hist_file_descriptor;
    int cum_file_descriptor;
    int height;
    int width;
    int num_points;
    double *input_data;
    double **input_data_matrix;
    double *hist_data;
    double **hist_data_matrix;
    double *cum_data;
    double **cum_data_matrix;
    int row;
    int col;
    int *histogram;
    int *cumulative;
    int level;
    int max_hist;
    int max_cum;
    int fill_hist;
    int fill_cum;

    if(argc < 4) {
	(void) fprintf(stdout, 
	"Syntax: showhistogram image height width\n");
	(void) fflush(stdout);
	(void) exit(0);
    }
    (void) sscanf(argv[1], "%s", input_file_name);
    (void) sscanf(argv[2], "%d", &height);
    (void) sscanf(argv[3], "%d", &width);
    (void) strcpy(hist_file_name, input_file_name);
    (void) strcat(hist_file_name, ".hist");
    (void) strcpy(cum_file_name, input_file_name);
    (void) strcat(cum_file_name, ".cum");
    input_file_descriptor = (int) open(input_file_name, O_RDONLY, 0);
    hist_file_descriptor = (int) creat(hist_file_name, 0644);
    cum_file_descriptor = (int) creat(cum_file_name, 0644);
    if((num_points = (int) lseek(input_file_descriptor, 0, SEEK_END)) ==
    (int) NIL) {
	(void) fprintf(stdout, "Error: File %s nonexistent.  Exit 1.\n",
	input_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) lseek(input_file_descriptor, -(num_points), SEEK_END);
    input_data = (double *) RealCreateVector(num_points);
    input_data_matrix = (double **) RealCreateMatrix(height, width);
    hist_data = (double *) RealCreateVector(HEIGHT * WIDTH);
    hist_data_matrix = (double **) RealCreateMatrix(HEIGHT, WIDTH);
    cum_data = (double *) RealCreateVector(HEIGHT * WIDTH);
    cum_data_matrix = (double **) RealCreateMatrix(HEIGHT, WIDTH);
    (void) ReadIn(input_file_descriptor, input_data, input_data_matrix,
    NOVERTICALNOHORIZONTALFLIP, height, width, num_points);
    histogram = (int *) SafeMalloc((unsigned) NUMGRAYS * sizeof(int),
    "histogram", "main");
    cumulative = (int *) SafeMalloc((unsigned) NUMGRAYS * sizeof(int),
    "cumulative", "main");
    for(level = 0; level < NUMGRAYS; level++) {
	histogram[level] = 0;
    }
    for(row = 0; row < height; row++) {
	for(col = 0; col < width; col++) {
	    (histogram
	    [(int)(input_data_matrix[row][col]-1.0*MINGRAYVALUE+EPSILON)])++;
	}
    }
    cumulative[0] = histogram[0];
    for(level = 1; level < NUMGRAYS; level++) {
	cumulative[level] = cumulative[level-1] + histogram[level];
    }
    max_hist = NEGHUGE;
    max_cum = NEGHUGE;
    for(level = 0; level < NUMGRAYS; level++) {
	max_hist = (int) MaxInt(max_hist, histogram[level]);
	max_cum = (int) MaxInt(max_cum, cumulative[level]);
    }
    for(level = 0; level < NUMGRAYS; level++) {
	fill_hist = (int) FixFloat((1.0 * HEIGHT * histogram[level]) /
	(max_hist + EPSILON));
	fill_cum = (int) FixFloat((1.0 * HEIGHT * cumulative[level]) /
	(max_cum + EPSILON));
	for(col = (BINSIZE * level); (col < ((BINSIZE * level) + BINSIZE));
	col++) {
	    for(row = 0; ((row < fill_hist) && (row < HEIGHT)); row++) {
		hist_data_matrix[row][col] = 1.0 * MAXGRAYVALUE;
	    }
	    for(row = 0; ((row < fill_cum) && (row < HEIGHT)); row++) {
		cum_data_matrix[row][col] = 1.0 * MAXGRAYVALUE;
	    }
	}
    }
    (void) RealMatrixToRealVector(hist_data, hist_data_matrix, HEIGHT, WIDTH);
    (void) WriteOut(hist_file_descriptor, VERTICALFLIP, 1, 
    hist_data, HEIGHT, WIDTH, HEIGHT * WIDTH, COEFF);
    (void) RealMatrixToRealVector(cum_data, cum_data_matrix, HEIGHT, WIDTH);
    (void) WriteOut(cum_file_descriptor, VERTICALFLIP, 1, 
    cum_data, HEIGHT, WIDTH, HEIGHT * WIDTH, COEFF);
    if((int) close(input_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, 
	"Error: Cannot close input file '%s'.  Exit 1.\n", input_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    if((int) close(hist_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, 
	"Error: Cannot close histogram file '%s'.  Exit 1.\n",
	hist_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    if((int) close(cum_file_descriptor) == (int) NIL) {
	(void) fprintf(stdout, 
	"Error: Cannot close cumulative file '%s'.  Exit 1.\n",
	cum_file_name);
	(void) fflush(stdout);
	(void) exit(1);
    }
    (void) RealFreeVector(input_data);
    (void) RealFreeMatrix(input_data_matrix, height);
    (void) RealFreeVector(hist_data);
    (void) RealFreeMatrix(hist_data_matrix, HEIGHT);
    (void) RealFreeVector(cum_data);
    (void) RealFreeMatrix(cum_data_matrix, HEIGHT);
    (void) free(histogram);
    (void) free(cumulative);
    (void) free(input_file_name);
    (void) free(hist_file_name);
    (void) free(cum_file_name);
    (void) exit(0);
}
