/*
*******************************************************************************
Author: 
	Alex Sherstinsky,
	Media Laboratory, 
	Building E15 Room 383,
	20 Ames Street, Cambridge, MA 02139,
	Massachusetts Institute Of Technology,
	77 Massachusetts Avenue,
	Cambridge, MA 02139
	Telephone: (617)253-0381 
	Electronic Mail Address: shers@media.mit.edu
Copyright 1993 Massachusetts Institute Of Technology
All rights reserved.
This program may not be distributed to other organizations without the 
permission of the author.  In no case may it be distributed outside the
United States and Canada.  This program is supplied "as is", without any
kind of warranty, and M.I.T. does not promise to support it in any way.
This program is not to be incorporated into a commercial product, or access
to it sold, without prior written agreement from the author.
*******************************************************************************
AlMa Library
*******************************************************************************
*/
#include "stdutils.h"
#include "func.h"

/*
Returns the sign of the integer.
*/

int
SignInt(int1)
int int1;

{
    return((int1 > 0) ? 1 : (-1));
}

/*
Returns the sign of the float.
*/

double
SignFloat(float1)
double float1;

{
    return((float1 > 0.0) ? 1.0 : (-1.0));
}

/*
Returns the minimum of two integers.
*/

int
MinInt(int1, int2)
int int1, int2;

{
    return((int1 < int2) ? int1 : int2);
}

/*
Returns the minimum of two floats.
*/

double
MinFloat(float1, float2)
double float1, float2;

{
    return((float1 < float2) ? float1 : float2);
}

/*
Returns the maximum of two integers.
*/

int 
MaxInt(int1, int2)
int int1, int2;

{
    return((int1 > int2) ? int1 : int2);
}

/*
Returns the maximum of two floats.
*/

double 
MaxFloat(float1, float2)
double float1, float2;

{
    return((float1 > float2) ? float1 : float2);
}

/*
Swaps two integers.
*/

void
SwapInt(int1, int2)
int *int1, *int2;

{
    int tmp;

    tmp = *int1;
    *int1 = *int2;
    *int2 = tmp;
}

/*
Swaps two floats.
*/

void
SwapFloat(float1, float2)
double *float1, *float2;

{
    double tmp;

    tmp = *float1;
    *float1 = *float2;
    *float2 = tmp;
}

/*
Returns absolute value of an integer.
*/

int
AbsInt(int1)
int int1;

{
    return((int1 < 0) ? (- int1) : int1);
}

/*
Returns absolute value of an float.
*/

double
AbsFloat(float1)
double float1;

{
    return((float1 < 0) ? (- float1) : float1);
}

/*
Returns the square of the input integer.
*/

int
SquareInt(int1)
int int1;

{
    return(int1 * int1);
}

/*
Returns the square of the input float.
*/

double
SquareFloat(float1)
double float1;

{
    return(float1 * float1);
}

/*
This function converts the given floating point number to the nearest integer.
*/

int
FixFloat(float1)
double float1;

{
    int int1;
    int int2;

    int1 = (int) float1;
    if(float1 >= 0.0) {
	int2 = int1 + 1;
	if(((int1 + int2) / 2.0) > float1) {
	    return(int1);
	} else {
	    return(int2);
	} 
    } else {
	int2 = int1 - 1;
	if(((int1 + int2) / 2.0) < float1) {
	    return(int1);
	} else {
	    return(int2);
	} 
    }
}

/*
Returns 1 if the argument is an Odd integer; 0 otherwise.
*/

int
Odd(int1)
int int1;

{
    return(int1 % 2);
}

/*
Returns 1 if the only one of the arguments is 1.
*/

int
Xor(int1, int2)
int int1;
int int2;

{
    return((int1 && !int2) || (!int1 && int2));
}

/*
Returns the middle of three strings that comprise the overall string.
*/

char *
MidString(line)
char *line;

{
    char *middle = (char *) SafeMalloc((unsigned) 121 * sizeof(char),
    "middle", "MidString");
    char first[20];
    char third[20];

    (void) sscanf(line, "%s %s %s", first, middle, third);
    return(middle);
}

/*
Allocates a string for a given number of characters and returns a pointer
to it.
*/

char *
AllocateString(num_chars)
int num_chars;

{
    char *string = 
    (char *) SafeMalloc((unsigned) num_chars * sizeof(char),
    "string", "AllocateString");
    return(string);
}

/*
This function is a modified pseudo random number generator.
*/

double
Random()

{
#define MODULUS (1 << 20)
    return(1.0 * ((int) random() % MODULUS) / MODULUS);
}

/*
This function implements N! for N >= 1.  Otherwise it returns 1.
*/

int
Factorial(n)
int n;

{
    if(n > 1) {
	return(n * (int) Factorial(n - 1));
    } else {
	return(1);
    }
}

/*
This function implements the combinatoric choose operation for N >= M.
*/

int
Choose(n, m)
int n;
int m;

{
    if(n >= m) {
	return((int) Factorial(n) / 
	((int) Factorial(m) * (int) Factorial(n - m)));
    } else {
	(void) fprintf(stdout, 
	"Error: Illegal parameters in 'Choose'.  Exit 1.\n");
	(void) fflush(stdout);
	(void) exit(1);
	return(NIL);
    }
}

/*
This function constructs the table of all (size) possibilities, in which M 
numbers (0 <= M <= N) can be chosen out of N numbers, ranging from 0 to N - 1.  
The [size by M] table of integers must be allocated previously.
*/

void 
ConstructChoiceTable(table, size, n, m)
int **table;
int size;
int n;
int m;

{
    int index;
    int row;
    int col;
    int prev;

    if(m > 0) {
	for(row = 0; row < size; row++) {
	    for(col = 0; col < m; col++) {
		table[row][col] = (int) NIL;
	    }
	}
	index = 0;
	(void) FindLegalChoices(table, &index, 0, 0, n, m);
	for(col = m - 1; col >= 0; col--) {
	    prev = table[size-1][col];
	    for(row = size - 2; row >= 0; row--) {
		if(table[row][col] == (int) NIL) {
		    table[row][col] = prev;
		}
		prev = table[row][col];
	    }
	}
    }
}

/*
This function recursively determines the legal choices (permutations) of M 
numbers, ranging from start to position + N - M + 1.  The numbers must not 
repeat; must be in the increasing order; and must not differ only by their 
positions.
*/

void 
FindLegalChoices(table, index, position, start, n, m)
int **table;
int *index;
int position;
int start;
int n;
int m;

{
    int number;
    int limit;

    limit = position + n - m + 1;
    if(position < (m - 1)) {
	for(number = start; number < limit; number++) {
	    (void) FindLegalChoices(table, index, 
	    position + 1, number + 1, n, m);
	    table[*index-1][position] = number;
	}
    } else {
	for(number = start; number < limit; number++) {
	    table[*index][position] = number;
	    (*index)++;
	}
    }
}

/*
This function attempts to allocate the requested number of bytes of memory and 
gives an error message if it fails.
*/

char *
SafeMalloc(num_bytes, var_name, func_name)
unsigned num_bytes;
char *var_name;
char *func_name;

{
    char *ptr;

    if((ptr = (char *) malloc(num_bytes)) == (char *) NULL) {
	(void) fprintf(stdout,
	"Error: No memory for variable '%s' in function '%s'.  Abort.\n",
	var_name, func_name);
	(void) fflush(stdout);
	(void) abort();
    }
    return(ptr);
}
